# -*- coding: utf-8 -*-

## 包含一些对队伍常用的操作

## slot
## 用于保存巡逻的据点
from smart_module.module_system.header_operations import *
from smart_module.module_system.header_parties import *
from smart_module.module_system.header_skills import *
from smart_module.module_system.module_constants import *
from smart_module.smart_core.smart_module_slot import smartModuleSlotManager

slot_party_protect_center = smartModuleSlotManager.getPartySlotNo("slot_party_protect_center")


## args
## 巡逻队类型
spt_patrol             = 7

## args


per_strength_xp = 100


partyBaseScripts={
    "name":"PartyBaseScripts",
    "enable":True,
    "version": "v2.0.0",
    "desc":"与部队或据点相关的操作",
    "actions":[
        ("Append@scripts",[
            ## 创建部队通用方法
            ("create_party",[
                ## 领导者
                (store_script_param, ":leader_no", 1),
                ## 创建在哪个地区附近
                (store_script_param, ":center_no", 2),
                ## 队伍阵营，优先使用领导者阵营，其次才会使用据点阵营，否则就是平民阵营
                (store_script_param, ":faction_no", 3),
                ## 范围
                (store_script_param, ":spawn_radius", 4),
                ## 队伍名称id（优先使用队伍领导者名称，其实使用据点名称，最后使用国家名称)
                (store_script_param, ":party_name", 5),
                ## 队伍类型
                (store_script_param, ":party_type", 6),
                ## 图标
                (store_script_param, ":map_icon", 7),
                ## 旗帜
                (store_script_param, ":map_banner", 8),

                (try_begin),
                    (lt,":center_no",0),
                    (assign,":center_no","p_main_party"),
                (try_end),

                (try_begin),
                    (lt,":faction_no",0),
                    (try_begin),
                        (ge,":leader_no",0),
                        (store_faction_of_troop,":faction_no",":leader_no"),
                    (else_try),
                        (ge,":center_no",0),
                        (store_faction_of_party,":faction_no",":center_no"),
                    (else_try),
                        (assign,":faction_no","fac_commoners"),
                    (try_end),
                (try_end),

                (try_begin),
                    (lt,":spawn_radius",0),
                    (assign,":spawn_radius",3),
                (try_end),

                (set_spawn_radius, ":spawn_radius"),
                (spawn_around_party, ":center_no", "pt_kingdom_hero_party"),
                (assign, ":new_party", reg0),
                ## 设置阵营
                (party_set_faction, ":new_party", ":faction_no"),
                (party_set_flags, ":new_party", pf_default_behavior, 0),

                (try_begin),
                    (ge,":leader_no",0),
                    ## 设置领导的队伍
                    (troop_set_slot,":leader_no",slot_troop_leaded_party,":new_party"),
                    ## 添加领导者
                    (party_add_leader,":new_party",":leader_no"),
                (try_end),

                (try_begin),
                    (ge,":leader_no",0),
                    (str_store_troop_name,s5,":leader_no"),
                (else_try),
                    (ge,":center_no",0),
                    (str_store_party_name,s5,":center_no"),
                (else_try),
                    (ge,":faction_no",0),
                    (str_store_faction_name,s5,":faction_no"),
                (try_end),

                ## 设置队伍名称
                (try_begin),
                (ge,":party_name",0),
                    (party_set_name, ":new_party", ":party_name"),
                (else_try),
                    (party_set_name, ":new_party", "str_s5_s_party"),
                (try_end),

                (try_begin),
                    (ge,":party_type",0),
                    (party_set_slot,":new_party",slot_party_type,":party_type"),
                (try_end),

                ## 设置指定外观
                (try_begin),
                    (ge,":map_icon",0),
                    (party_set_icon,":new_party",":map_icon"),
                (try_end),

                ## 设置指定旗帜
                (try_begin),
                    (ge,":map_banner",0),
                    (party_set_banner_icon,":new_party",":map_banner"),
                (try_end),
                (assign, reg0, ":new_party"),
              ]),

            ("party_add_members", [
                (store_script_param, ":party_no",1),
                (store_script_param, ":faction_no",2),
                ## 军事强度
                (store_script_param, ":strength",3),
                ## 队伍模板，
                ## -1,使用阵营士兵模板
                ## 指定模板时，使用指定模板添加士兵
                (store_script_param, ":party_template", 4),
                ## 初级士兵概率（值最大为100）
                (store_script_param, ":primary_probability",5),
                ## 中级士兵概率（值最大为100）
                (store_script_param, ":intermediate_probability",6),
                ## 高级士兵概念（高级 = 100 - 初级 - 中级）
                #(store_script_param, ":advanced_probability",6),

                (try_begin),
                    (lt,":faction_no",0),
                    (store_faction_of_party,":faction_no",":party_no"),
                (try_end),

                (try_begin),
                    (le,":strength",0),
                    (assign,":strength",1),
                (try_end),

                ## 处理模板
                (try_begin),
                    ## 默认模板时，使用阵营士兵模板
                    (lt,":party_template",0),
                    ## 计算中级士兵的概率
                    (val_add,":intermediate_probability",":primary_probability"),
                    ## 根据概率获得阵营模板
                    (store_random_in_range, ":pt_no", 0, 100),
                    (try_begin),
                        (le, ":pt_no", ":primary_probability"),
                        (faction_get_slot, ":pt_temp", ":faction_no", slot_faction_reinforcements_a),
                    (else_try),
                        (le, ":pt_no", ":intermediate_probability"),
                        (faction_get_slot, ":pt_temp", ":faction_no", slot_faction_reinforcements_b),
                    (else_try),
                        (faction_get_slot, ":pt_temp", ":faction_no", slot_faction_reinforcements_c),
                    (try_end),
                (else_try),
                    ## 使用指定模板
                    (assign,":pt_temp",":party_template"),
                (try_end),
                ## 根据模板生成士兵
                (try_for_range, ":unused", 0, ":strength"),
                    (party_add_template, ":party_no", ":pt_temp"),
                (try_end),
            ]),
            ("party_add_xp_and_upgrade",[
                (store_script_param, ":party_no",1),
                (store_script_param, ":strength",2),
                (store_script_param, ":special_arms_probability",3),

                (try_begin),
                    (gt,":strength",0),
                    ## 计算军事强度(同伴有教练技能时会增加经验)
                    (party_get_num_companion_stacks,":stack",":party_no"),
                    (try_for_range,":stack_index",0,":stack"),
                        (party_stack_get_troop_id,":troop_no",":party_no",":stack_index"),
                        (troop_is_hero,":troop_no"),
                        (store_skill_level,":trainer_level",":troop_no",skl_trainer),
                        (val_add,":strength",":trainer_level"),
                    (try_end),
                    (store_mul, ":total_xp", ":strength", per_strength_xp),

                    ## 根据特殊兵种概率来升级更加强壮的士兵，如果没有指定概率就是随机升级
                    (try_begin),
                        (gt,":special_arms_probability",0),
                        (store_random_in_range,":pro",0,100),
                        (try_begin),
                            (le,":special_arms_probability",":pro"),
                            (party_upgrade_with_xp, ":party_no", ":total_xp", 1),
                        (else_try),
                            (party_upgrade_with_xp, ":party_no", ":total_xp", 2),
                        (try_end),
                    (else_try),
                        (party_upgrade_with_xp, ":party_no", ":total_xp", 0),
                    (try_end),
                (try_end),
            ]),
            ("party_change_ai_state",[
                (store_script_param,":party",1),
                (store_script_param,":behavior",2),
                (store_script_param,":target",3),
                (store_script_param,":patrol_radius",3),

                (try_begin),
                    (le,":patrol_radius",0),
                    (assign,":patrol_radius",3),
                (try_end),

                # (party_set_slot, ":party", slot_party_ai_state, spai_patrolling_around_center),
                # (party_set_slot, ":party", slot_party_ai_object, ":target"),
                (party_set_ai_behavior, ":party", ":behavior"),
                (party_set_ai_object, ":party", ":target"),
                (party_set_ai_patrol_radius, ":party", ":patrol_radius"),
            ]),
            ## 计算俘虏价格
            ("get_prisoner_prices",[
                 (store_script_param_1, ":troop_id"),
                 (store_script_param_2, ":size"),
                 (store_character_level, ":troop_level", ":troop_id"),
                 (assign, ":ransom_amount", ":troop_level"),
                 (val_add, ":ransom_amount", 10),
                 (val_mul, ":ransom_amount", ":ransom_amount"),
                 (val_div, ":ransom_amount", 6),
                 (val_mul,":ransom_amount",":size"),
                 (assign, reg0, ":ransom_amount"),
             ]),
            ## 更新据点财富
            ("update_center_wealth",[
                 (store_script_param_1, ":center_no"),
                 (store_script_param_2, ":value"),
                 ## 0:失去钱 1：获得钱
                 (store_script_param, ":type",3),

                 (troop_get_slot,":wealth",":center_no",slot_town_wealth),
                 (try_begin),
                     (gt,":type",0),
                     (val_add,":wealth",":value"),
                 (else_try),
                    (val_sub,":wealth",":value"),
                 (try_end),
                 (troop_set_slot,":center_no",slot_town_wealth,":wealth"),
             ]),
            ("party_handle_prisoners",[
                ## 需要处理俘虏的队伍
                (store_script_param_1,":party_no"),
                ## 是否会招募俘虏  -1，全部售卖 0:招募本国士兵 1：招募全部士兵
                (store_script_param_2,":recruit"),
                (party_get_num_prisoners,":pri_size",":party_no"),
                ## 有俘虏才进行处理
                (gt,":pri_size",0),

                (store_faction_of_party,":party_faction",":party_no"),
                (party_get_num_prisoner_stacks,":stack",":party_no"),
                (assign,":total_price",0),
                (try_for_range_backwards,":index",0,":stack"),
                    (party_prisoner_stack_get_troop_id,":troop",":party_no",":index"),
                    (store_faction_of_troop,":troop_faction",":troop"),
                    ## 本栏中俘虏的个数
                    (party_prisoner_stack_get_size,":cur_stack_size",":party_no",":index"),
                    (try_begin),
                        ## 全部售卖
                        (eq,":recruit",-1),
                        (call_script, "script_get_prisoner_prices", ":troop", ":cur_stack_size"),
                        (assign, ":pri_price", reg0),
                        (val_add, ":total_price", ":pri_price"),
                        #(display_message,"@出售全部士兵"),
                    (else_try),
                        ## 本国招募，它国售卖
                        (eq, ":recruit", 0),
                        (try_begin),
                            (str_store_faction_name,s1,":party_faction"),
                            (str_store_faction_name,s2,":troop_faction"),
                            (str_store_troop_name,s3,":troop"),
                            #(display_message,"@party faction name ({s1})     troop faction name ({s2} troop:{s3})"),
                            ## 本国招募
                            (eq,":party_faction",":troop_faction"),
                            ## 添加同伴
                            (party_add_members,":party_no",":troop",":cur_stack_size"),
                            # (str_store_party_name,s1,":party_no"),
                            # (str_store_troop_name,s2,":troop"),
                            # (assign,reg1,":cur_stack_size"),
                            #(display_message,"@do add troops({reg1}):{s1}"),
                            #(display_message,"@招募本国士兵"),
                        (else_try),
                            ## 它国贩卖
                            (call_script,"script_get_prisoner_prices",":troop",":cur_stack_size"),
                            (assign,":pri_price",reg0),
                            (val_add,":total_price",":pri_price"),
                            #(display_message,"@售卖它国部分士兵"),
                        (try_end),
                    (else_try),
                        ## 全部招募
                        (party_add_members, ":party_no", ":troop", ":cur_stack_size"),
                        #(display_message,"@招募全部士兵"),
                    (try_end),
                    ## 将处理后的俘虏从俘虏栏中移除
                    (party_remove_prisoners,":party_no",":troop",":cur_stack_size"),
                    #(str_store_party_name,s1,":party_no"),
                    #(display_message,"@do remove prisoners:{s1}"),

                (try_end),
                (gt,":total_price",0),
                (party_stack_get_troop_id,":leader",":party_no",0),
                (try_begin),
                    (ge,":leader",0),
                    (call_script, "script_update_lord_wealth", ":leader", ":total_price", 1),
                (else_try),
                    (party_slot_eq,":party_no",slot_party_type,spt_patrol),
                    (party_get_slot,":center",":party_no",slot_party_protect_center),
                    (call_script,"script_update_center_wealth",":center",":total_price",1),
                (else_try),
                    (store_faction_of_party,":faction",":party_no"),
                    (faction_get_slot,":king",":faction",slot_faction_leader),
                    (call_script, "script_update_lord_wealth", ":king", ":total_price", 1),
                (try_end),
            ]),
            ## 获得指定队伍附近指定类型的队伍
            ("get_the_nearby_center",[
                ## 当前队伍
                (store_script_param_1,":cur_party"),
                ## 最近队伍的类型(如：附近的村庄（spt_village），附近的城堡（spt_castle），附近的城镇（spt_town）)
                (store_script_param_2,":party_type"),
                (assign,":min",99999999),
                (assign,":close_center",":cur_party"),
                (try_for_range,":center_no",centers_begin,centers_end),
                    ## 过滤指定类型
                    (party_slot_eq,":center_no",slot_party_type,":party_type"),
                    ## 获得据点之间的距离
                    (store_distance_to_party_from_party,":distance",":center_no",":cur_party"),
                    (le,":distance",":min"),
                    (assign,":min",":distance"),
                    (assign,":close_center",":center_no"),
                (try_end),
                (assign,reg0,":close_center"),
            ]),
            ## 获得一个阵营全部的据点
            ("get_all_center_arr_of_faction", [
                (store_script_param_1, ":faction"),
                (assign, ":size", 0),
                (assign, ":index", 0),
                (try_for_range, ":center", centers_begin, centers_end),
                (this_or_next | party_slot_eq, ":center", slot_party_type, spt_town),
                (this_or_next | party_slot_eq, ":center", slot_party_type, spt_castle),
                (party_slot_eq, ":center", slot_party_type, spt_village),
                (store_faction_of_party, ":party_faction", ":center"),
                (eq, ":party_faction", ":faction"),
                # (store_sub,":offset",":center",centers_begin),
                ## 跳过0下标，下标为0的位置用于存放据点的个数
                (val_add, ":index", 1),
                (party_set_slot, "p_temp_party", ":index", ":center"),
                (val_add, ":size", 1),
                (try_end),
                (party_set_slot, "p_temp_party", 0, ":size"),
            ]),
            ("add_party_as_companions",
                [
                  (store_script_param, ":target_party",1), #Target Party_id
                  (store_script_param, ":source_party",2), #Source Party_id
                  ## 英雄是否也加入： -1代表只加入士兵，大于等于0代表英雄加入
                  (store_script_param, ":include_hero",3), #Source Party_id
                  (party_get_num_companion_stacks, ":num_stacks",":source_party"),
                  (try_for_range, ":stack_no", 0, ":num_stacks"),
                    (party_stack_get_troop_id, ":stack_troop",":source_party",":stack_no"),
                    (assign,":hero_can_join",0),
                    (try_begin),
                        (troop_is_hero,":stack_troop"),
                        (ge,":include_hero",0),
                        (assign,":hero_can_join",1),
                    (try_end),
                    ## 非英雄
                    (this_or_next|neg|troop_is_hero,":stack_troop"),
                    ## 或者 是英雄，也允许英雄加入
                    (eq,":hero_can_join",1),
                    (party_stack_get_size, ":stack_size",":source_party",":stack_no"),
                    (party_add_members, ":target_party", ":stack_troop", ":stack_size"),

                    (party_stack_get_num_wounded, ":num_wounded", ":source_party", ":stack_no"),
                    (party_wound_members, ":target_party", ":stack_troop", ":num_wounded"),

                    (str_store_troop_name,s6,":stack_troop"),
                    (try_begin),
                        (troop_is_hero,":stack_troop"),
                        (display_message, "@{s6} has joined your party."),
                    (else_try),
                        (assign,reg1,":stack_size"),
                        (display_message, "@{reg1} {s6} join your party."),
                    (try_end),
                  (try_end),
                  (remove_party,":source_party"),
              ]),

            ("get_loard_all_centers",[
                (store_script_param_1,":target_troop"),
                (store_script_param_2,":source_troop"),
                (try_for_range,":center",centers_begin,centers_end),
                    (party_slot_eq,":center",slot_town_lord,":source_troop"),
                    (call_script,"script_give_center_to_lord",":center",":target_troop",1),
                (try_end),
            ]),
            ("get_center_of_lord",[
                (store_script_param_1,":lord"),
                (store_script_param_2,":default_center"),
                (assign,":town",":default_center"),
                (assign,":castle",":default_center"),
                (assign,":village",":default_center"),
                (try_for_range,":center",centers_begin,castles_end),
                    (party_slot_eq,":center",slot_town_lord,":lord"),
                    (try_begin),
                        (party_slot_eq,":center",slot_party_type,spt_town),
                        (assign,":town",":center"),
                    (else_try),
                        (party_slot_eq,":center",slot_party_type,spt_castle),
                        (assign,":castle",":center"),
                    (else_try),
                        (party_slot_eq,":center",slot_party_type,spt_village),
                        (assign,":village",":center"),
                    (try_end),
                (try_end),

                (try_begin),
                    (ge,":town",0),
                    (assign,reg0,":town"),
                (else_try),
                    (ge, ":castle", 0),
                    (assign, reg0, ":castle"),
                (else_try),
                    (ge, ":village", 0),
                    (assign, reg0, ":village"),
                (try_end),
            ]),
            ("get_party_max_skill",[
                (store_script_param_1,":party_no"),
                (store_script_param_2,":skill_id"),
                (assign,":max_level",0),
                (party_get_num_companion_stacks,":stacks",":party_no"),
                (try_for_range,":stack_i",0,":stacks"),
                    (party_stack_get_troop_id,":troop_no",":party_no",":stack_i"),
                    (troop_is_hero,":troop_no"),
                    (store_skill_level,":skill_level",":troop_no",":skill_id"),
                    (gt,":skill_level",":max_level"),
                    (assign,":max_level",":skill_level"),
                (try_end),
                (assign,reg0,":max_level"),
            ]),
        ])
    ],
    "internationals":{
        "cns":{
            "game_strings":
                [
                    "str_s5_leave_home|{s5}出 门 了",
                    "str_s5_go_home|{s5}回 家 了",
                    "str_s1_s2_name|{s1}{s2}",
                ],
            "quick_strings":[
                "qstr_{reg1}_{s6}_join_your|{reg1} 个 {s6} 加 入 到 你 的 了 队 伍。",
            ],
        }
    }
}


###################################################
# header_tableau_materials.py
# This file contains declarations for tableau materials
# DO NOT EDIT THIS FILE!
###################################################

from smart_module.module_system.header_common import *
from smart_module.module_system.header_operations import *




from smart_module.module_system.process_operations import *


def save_scripts(variable_list,variable_uses,scripts,tag_uses,quick_strings,export_dir):
  file = open(export_dir + "scripts.txt","w",encoding="utf-8")
  file.write("scriptsfile version 1\n")
  file.write("%d\n"%len(scripts))
  temp_list = []
  list_type = type(temp_list)
  for i_script in range(len(scripts)):
    func = scripts[i_script]
    if (type(func[1]) == list_type):
      file.write("%s -1\n"%(convert_to_identifier(func[0])))
      save_statement_block(file,convert_to_identifier(func[0]), 0,func[1], variable_list,variable_uses,tag_uses,quick_strings)
    else:
      file.write("%s %f\n"%(convert_to_identifier(func[0]), func[1]))
      save_statement_block(file,convert_to_identifier(func[0]), 0,func[2], variable_list,variable_uses,tag_uses,quick_strings)
    file.write("\n")
  file.close()

def save_python_header(src_dir):
  file = open(src_dir + "ID_scripts.py","w",encoding="utf-8")
  for i_script in range(len(scripts)):
    file.write("script_%s = %d\n"%(convert_to_identifier(scripts[i_script][0]),i_script))
  file.write("\n\n")
  file.close()


def processScripts(context):
  print("Exporting scripts...")

  configParser = context.configParser
  src_dir = configParser.getSrcDir()
  export_dir = configParser.getExportDir()

  save_python_header(src_dir)
  variable_uses = []
  variables = load_variables(export_dir, variable_uses)
  tag_uses = load_tag_uses(export_dir)
  quick_strings = load_quick_strings(export_dir)
  save_scripts(variables, variable_uses, scripts, tag_uses, quick_strings,export_dir)
  save_variables(export_dir, variables, variable_uses)
  save_tag_uses(export_dir, tag_uses)
  save_quick_strings(export_dir, quick_strings)




from smart_module.module_system.module_party_templates import *
#from process_operations import *

from smart_module.module_system.process_common import *

def save_party_template_troop(file,troop):
  if troop:
#    add_tag_use(tag_uses,tag_troop,troop[0])
    file.write("%d %d %d "%(troop[0],troop[1],troop[2]))
    if (len(troop) > 3):
      file.write("%d "%troop[3])
    else:
      file.write("0 ")
  else:
    file.write("-1 ")
    
def save_party_templates(export_dir):
  file = open(export_dir + "party_templates.txt","w",encoding="utf-8")
  file.write("partytemplatesfile version 1\n")
  file.write("%d\n"%(len(party_templates)))
  for party_template in party_templates:
#    add_tag_use(tag_uses,tag_faction,party_template[4])
    file.write("pt_%s %s %d %d %d %d "%(convert_to_identifier(party_template[0]),replace_spaces(party_template[1]),party_template[2],party_template[3], party_template[4], party_template[5]))
    members = party_template[6]
    if (len(members) > 6):
      print( "Error! NUMBER OF TEMPLATE MEMBERS EXCEEDS 6 " + party_template[0])
      members = members[0:6]
    for party_template_member in members:
      save_party_template_troop(file,party_template_member)
    for i in range(6 - len(members)):
      save_party_template_troop(file,0)
    file.write("\n")
  file.close()

def save_python_header(src_dir):
  file = open(src_dir + "ID_party_templates.py","w",encoding="utf-8")
  for i_party_template in range(len(party_templates)):
    file.write("pt_%s = %d\n"%(convert_to_identifier(party_templates[i_party_template][0]),i_party_template))
  file.close()


def processPartyTmps(context):
    print( "Exporting party_template data...")

    configParser = context.configParser
    src_dir = configParser.getSrcDir()
    export_dir = configParser.getExportDir()

    #tag_uses = load_tag_uses(export_dir)
    save_python_header(src_dir)
    save_party_templates(export_dir)
    #save_tag_uses(export_dir, tag_uses)



from smart_module.module_system.ID_troops import *
from smart_module.module_system.header_dialogs import *
from smart_module.module_system.header_game_menus import *
from smart_module.module_system.module_mission_templates import *

testMode = {
    "name":"testMode",
    "enable":True,
    "version": "v2.0.0",
    "desc":"与测试相关的功能，各种常用场景，进入大地图，进入村庄，进入战场等等",
    "actions":[
        ("GameStartMenu",[
            ("test_map",[],"TEST 【map】",
             [
                 (call_script,"script_go_to_center",-1,-1,100000,5000,1,1,1),
            ]),
            ("test_village",[],"TEST 【village】",
             [
                 (call_script,"script_go_to_center","p_village_1",-1,100000,5000,1,-1,-1),
                 (call_script,"script_init_village_base_basic_business_data"),
            ]),
            ("test_alley",[],"TEST 【alley】",
             [
                 (call_script,"script_create_battle_for_player","p_town_1",slot_town_alley,20,20),
             ]),

             ("test_arena",[],"TEST 【arena】",
             [
                 (call_script,"script_create_battle_for_player","p_town_1",slot_town_arena,20,20),
             ]),

            ("test_tavern",[],"TEST 【tavern】",
             [
                 (call_script,"script_create_battle_for_player","p_town_1",slot_town_tavern,20,20),
             ]),

            ("test_prison",[],"TEST 【prison】",
             [
                 (call_script,"script_create_battle_for_player","p_town_1",slot_town_prison,20,20),
             ]),
            ("test_castle",[],"TEST 【castle】",
             [
                 (call_script,"script_create_battle_for_player","p_town_1",slot_town_castle,20,20),
             ]),

            ("test_center",[],"TEST 【center】",
             [
                 (call_script,"script_create_battle_for_player","p_town_1",slot_town_center,20,20),
             ]),

            ("test_plain",[],"TEST 【plain】",
             [
                 (call_script,"script_create_battle_for_player","scn_random_scene_plain",-1,20,20),
            ]),

            ("test_zendar",[],"TEST 【zendar】",
             [
                (set_visitor, 0, "trp_player"),
                (set_jump_mission, "mt_town_center"),
                (jump_to_scene, "scn_zendar_center"),


                (troop_set_name,"trp_player","@test"),
                (party_set_name,"p_main_party","@test"),
                (troop_add_item, "trp_player", "itm_saddle_horse", 0),
                (troop_add_item, "trp_player", "itm_courser", 0),
                (troop_add_item, "trp_player", "itm_courtly_outfit", 0),
                (troop_add_item, "trp_player", "itm_heraldic_mail_with_tabard", 0),
                (troop_add_item, "trp_player", "itm_red_gambeson", 0),
                (troop_add_item, "trp_player", "itm_sword_medieval_c", 0),
                (troop_add_item, "trp_player", "itm_tab_shield_kite_cav_b", 0),
                (troop_add_item, "trp_player", "itm_light_lance", 0),

                 (troop_raise_skill,"trp_player",skl_riding,10),
                 (troop_raise_skill,"trp_player",skl_leadership,10),

                (troop_raise_attribute,"trp_player",sf_base_att_str,10),
                (troop_raise_attribute,"trp_player",sf_base_att_agi,10),
                (troop_raise_attribute,"trp_player",sf_base_att_int,10),
                (troop_raise_attribute,"trp_player",sf_base_att_cha,10),

                (troop_add_items,"trp_player","itm_dried_meat",10),
                 # (try_for_range,":npc","trp_npc1","trp_npc16"),
                 #    (call_script,"script_recruit_troop_as_companion",":npc"),
                 #    (troop_raise_skill,":npc",skl_persuasion,1),
                 # (try_end),

                 (party_add_members,"p_main_party","trp_swadian_knight",70),
                 (troop_add_gold,"trp_player",100000),
                (troop_set_slot,"trp_player",slot_troop_renown,5000),

                (troop_raise_skill,"trp_player",skl_prisoner_management,10),
                (party_add_prisoners, "p_main_party", "trp_swadian_knight", 10),



                (change_screen_mission),
            ]),
        ]),
        ("Prepend@dialogs|start&1:Surrender_or_die%^&3:battle_reason_stated&4",[
            [trp_constable_hareck, "start", [], "what can I do for you?", "constable_hareck_hi",[]],

            [anyone | plyr, "constable_hareck_hi", [], "I want to be a king.", "constable_hareck_to_be_king", []],
            ## 选择国王
            [anyone, "constable_hareck_to_be_king", [], "who you want to be?", "constable_hareck_to_be_king_choice_king", []],
            [anyone | plyr | repeat_for_troops, "constable_hareck_to_be_king_choice_king", [
                (store_repeat_object,":king"),
                (is_between,":king",kings_begin,kings_end),
                (store_faction_of_troop,":faction",":king"),
                (str_store_faction_name,s1,":faction"),
                (str_store_troop_name,s2,":king"),
            ], "{s1} {s2}", "constable_hareck_to_be_king_choice_king_end", [
                (store_repeat_object,":king"),
                (assign,reg1,":king"),
            ]],
            [anyone, "constable_hareck_to_be_king_choice_king_end", [
                (assign,":king",reg1),
                (store_faction_of_troop,":faction",":king"),
                (str_store_faction_name,s1,":faction"),
                (str_store_troop_name,s2,":king"),
            ], "now,you are the king {s2} of faction {s1}.", "close_window", [
                (assign,":king",reg1),
                (call_script,"script_player_cosplay_anyone",":king"),
                (change_screen_map),
            ]],


            [anyone | plyr, "constable_hareck_hi", [], "I want to be a lord.", "constable_hareck_to_be_lord", []],
            ## 选择阵营
            [anyone, "constable_hareck_to_be_lord", [], "which faction you want to be lord?", "constable_hareck_to_be_lord_choice_faction", []],
            [anyone | plyr | repeat_for_factions, "constable_hareck_to_be_lord_choice_faction", [
                (store_repeat_object,":faction"),
                (is_between,":faction",npc_kingdoms_begin,npc_kingdoms_end),
                (str_store_faction_name,s1,":faction"),
            ], "{s1}", "constable_hareck_to_be_lord_choice_faction_end", [
                (store_repeat_object,":faction"),
                (assign,reg1,":faction"),
            ]],
            ## 选择领主
            [anyone, "constable_hareck_to_be_lord_choice_faction_end", [], "who you want to be?", "constable_hareck_to_be_lord_choice_lord", []],
            [anyone | plyr | repeat_for_troops, "constable_hareck_to_be_lord_choice_lord", [
                (store_repeat_object,":lord"),
                (is_between,":lord",lords_begin,lords_end),
                (store_faction_of_troop,":faction",":lord"),
                (eq,":faction",reg1),
                (str_store_troop_name,s1,":lord"),
            ], "{s1}", "constable_hareck_to_be_lord_choice_lord_end", [
                (store_repeat_object,":lord"),
                (assign,reg2,":lord"),
            ]],
            [anyone, "constable_hareck_to_be_lord_choice_lord_end", [
                (assign,":faction",reg1),
                (assign,":lord",reg2),
                (str_store_faction_name,s1,":faction"),
                (str_store_troop_name,s2,":lord"),
            ], "now,you are the {s2} lord of {s1} faction", "close_window", [
                (assign,":lord",reg2),
                (call_script,"script_player_cosplay_anyone",":lord"),
                (change_screen_map),
            ]],

            [anyone | plyr, "constable_hareck_hi", [], "never mind!", "close_window", []],
        ])
    ],
    "internationals":{
        "cns":{
            "game_menus":[
                "mno_test_alley|测 试 【 街 道 】",
                "mno_test_map|测 试 【 大 地 图 】",
                "mno_test_village|测 试 【 村 庄 】",
                "mno_test_arena|测 试 【 竞 技 场 】",
                "mno_test_tavern|测 试 【 酒 馆 】",
                "mno_test_prison|测 试 【 监 狱 】",
                "mno_test_castle|测 试 【 大 厅 】",
                "mno_test_center|测 试 【 地 形 】",
                "mno_test_zendar|测 试 【 禅 达 】",
            ],
            "dialogs":[
                "dlga_start:constable_hareck_hi|有 什 么 可 以 帮 助 你 的？",
                "dlga_constable_hareck_hi:constable_hareck_to_be_king|我 想 做 国 王",
                "dlga_constable_hareck_to_be_king:constable_hareck_to_be_king_choice_king|你 想 成 为 哪 个 国 王？",
                "dlga_constable_hareck_to_be_king_choice_king_end:close_window|现 在, 你 已 经 是 {s1} 的 {s2} 国 王 了!",
                "dlga_constable_hareck_hi:constable_hareck_to_be_lord|我 想 做 领 主",
                "dlga_constable_hareck_to_be_lord:constable_hareck_to_be_lord_choice_faction|你 想 成 为 哪 个 国 家 的 领 主?",
                "dlga_constable_hareck_to_be_lord_choice_faction_end:constable_hareck_to_be_lord_choice_lord|你 想 成 为 这 个 国 家 的 哪 位 领 主?",
                "dlga_constable_hareck_to_be_lord_choice_lord_end:close_window|现 在, 你 已 经 是 {s1} 的 {s2} 领 主 了!",

                "dlga_constable_hareck_hi:close_window|不 需 要",
            ]
        }
    },
}


from smart_module.module_system.header_music import *
####################################################################################################################
#  Each track record contains the following fields:
#  1) Track id: used for referencing tracks.
#  2) Track file: filename of the track
#  3) Track flags. See header_music.py for a list of available flags
#  4) Continue Track flags: Shows in which situations or cultures the track can continue playing. See header_music.py for a list of available flags
####################################################################################################################

# WARNING: You MUST add mtf_module_track flag to the flags of the tracks located under module directory

tracks = [
##  ("losing_battle", "alosingbattle.mp3", sit_calm|sit_action),
##  ("reluctant_hero", "reluctanthero.mp3", sit_action),
##  ("the_great_hall", "thegreathall.mp3", sit_calm),
##  ("requiem", "requiem.mp3", sit_calm),
##  ("silent_intruder", "silentintruder.mp3", sit_calm|sit_action),
##  ("the_pilgrimage", "thepilgrimage.mp3", sit_calm),
##  ("ambushed", "ambushed.mp3", sit_action),
##  ("triumph", "triumph.mp3", sit_action),

##  ("losing_battle", "alosingbattle.mp3", mtf_sit_map_travel|mtf_sit_attack),
##  ("reluctant_hero", "reluctanthero.mp3", mtf_sit_attack),
##  ("the_great_hall", "thegreathall.mp3", mtf_sit_map_travel),
##  ("requiem", "requiem.mp3", mtf_sit_map_travel),
##  ("silent_intruder", "silentintruder.mp3", mtf_sit_map_travel|mtf_sit_attack),
##  ("the_pilgrimage", "thepilgrimage.mp3", mtf_sit_map_travel),
##  ("ambushed", "ambushed.mp3", mtf_sit_attack),
##  ("triumph", "triumph.mp3", mtf_sit_attack),
  ("bogus", "cant_find_this.ogg", 0, 0),
  ("mount_and_blade_title_screen", "mount_and_blade_title_screen.ogg", mtf_sit_main_title|mtf_start_immediately, 0),

  ("ambushed_by_neutral", "ambushed_by_neutral.ogg", mtf_sit_ambushed|mtf_sit_siege, mtf_sit_fight|mtf_sit_multiplayer_fight),
  ("ambushed_by_khergit", "ambushed_by_khergit.ogg", mtf_culture_3|mtf_sit_ambushed|mtf_sit_siege, mtf_sit_fight|mtf_sit_multiplayer_fight|mtf_culture_all),
  ("ambushed_by_nord",    "ambushed_by_nord.ogg", mtf_culture_4|mtf_sit_ambushed|mtf_sit_siege, mtf_sit_fight|mtf_sit_multiplayer_fight|mtf_culture_all),
  ("ambushed_by_rhodok",  "ambushed_by_rhodok.ogg", mtf_culture_5|mtf_sit_ambushed|mtf_sit_siege, mtf_sit_fight|mtf_sit_multiplayer_fight|mtf_culture_all),
  ("ambushed_by_swadian", "ambushed_by_swadian.ogg", mtf_culture_1|mtf_sit_ambushed|mtf_sit_siege, mtf_sit_fight|mtf_sit_multiplayer_fight|mtf_culture_all),
  ("ambushed_by_vaegir",  "ambushed_by_vaegir.ogg", mtf_culture_2|mtf_sit_ambushed|mtf_sit_siege, mtf_sit_fight|mtf_sit_multiplayer_fight|mtf_culture_all),
  ("ambushed_by_sarranid", "middle_eastern_action.ogg", mtf_culture_6|mtf_sit_ambushed|mtf_sit_siege, mtf_sit_fight|mtf_sit_multiplayer_fight|mtf_culture_all),
  
  ("arena_1", "arena_1.ogg", mtf_sit_arena, 0),
#  ("arena_2", "arena_2.ogg", mtf_looping|mtf_sit_arena, 0),
  ("armorer", "armorer.ogg", mtf_sit_travel, 0),
  ("bandit_fight", "bandit_fight.ogg", mtf_sit_fight|mtf_sit_multiplayer_fight|mtf_sit_ambushed, 0),

  ("calm_night_1", "calm_night_2.ogg", mtf_sit_night, mtf_sit_town|mtf_sit_tavern|mtf_sit_travel),
  ("captured", "capture.ogg", mtf_persist_until_finished, 0),
  ("defeated_by_neutral", "defeated_by_neutral.ogg",mtf_persist_until_finished|mtf_sit_killed, 0),
  ("defeated_by_neutral_2", "defeated_by_neutral_2.ogg", mtf_persist_until_finished|mtf_sit_killed, 0),
  ("defeated_by_neutral_3", "defeated_by_neutral_3.ogg", mtf_persist_until_finished|mtf_sit_killed, 0),

  ("empty_village", "empty_village.ogg", mtf_persist_until_finished, 0),
  ("encounter_hostile_nords", "encounter_hostile_nords.ogg", mtf_persist_until_finished|mtf_sit_encounter_hostile, 0),
  ("escape", "escape.ogg", mtf_persist_until_finished, 0),

  ("fight_1", "fight_1.ogg", mtf_sit_fight|mtf_sit_multiplayer_fight|mtf_sit_ambushed, 0),
  ("fight_2", "fight_2.ogg", mtf_sit_fight|mtf_sit_multiplayer_fight|mtf_sit_ambushed, 0),
  ("fight_3", "fight_3.ogg", mtf_sit_fight|mtf_sit_multiplayer_fight|mtf_sit_ambushed, 0),
  ("fight_as_khergit", "fight_as_khergit.ogg", mtf_culture_3|mtf_sit_fight|mtf_sit_multiplayer_fight|mtf_sit_ambushed, mtf_culture_all),
  ("fight_4", "percussion_battery.ogg", mtf_sit_fight|mtf_sit_multiplayer_fight|mtf_sit_ambushed, 0),  
  ("fight_as_nord", "fight_as_nord.ogg", mtf_culture_4|mtf_sit_fight|mtf_sit_multiplayer_fight|mtf_sit_ambushed, mtf_culture_all),
  ("fight_as_rhodok", "fight_as_rhodok.ogg", mtf_culture_5|mtf_sit_fight|mtf_sit_multiplayer_fight|mtf_sit_ambushed, mtf_culture_all),
#  ("fight_as_swadian", "fight_as_swadian.ogg", mtf_culture_1|mtf_sit_fight|mtf_sit_multiplayer_fight, mtf_culture_all),
  ("fight_as_vaegir", "fight_as_vaegir.ogg", mtf_culture_2|mtf_sit_fight|mtf_sit_multiplayer_fight|mtf_sit_ambushed, mtf_culture_all),
  ("fight_while_mounted_1", "fight_while_mounted_1.ogg", mtf_sit_fight|mtf_sit_multiplayer_fight|mtf_sit_ambushed, 0),
  ("fight_while_mounted_2", "fight_while_mounted_2.ogg", mtf_sit_fight|mtf_sit_multiplayer_fight|mtf_sit_ambushed, 0),
  ("fight_while_mounted_3", "warband_action.ogg", mtf_sit_fight|mtf_sit_multiplayer_fight|mtf_sit_ambushed, 0),
  
  ("infiltration_khergit", "infiltration_khergit.ogg", mtf_culture_3|mtf_sit_town_infiltrate, mtf_culture_all),

  ("killed_by_khergit", "killed_by_khergit.ogg", mtf_persist_until_finished|mtf_culture_3|mtf_sit_killed, 0),
#  ("killed_by_neutral", "killed_by_neutral.ogg", mtf_persist_until_finished|mtf_culture_6|mtf_sit_killed, 0),
#  ("killed_by_nord", "killed_by_nord.ogg", mtf_persist_until_finished|mtf_culture_4|mtf_sit_killed, 0),
#  ("killed_by_rhodok", "killed_by_rhodok.ogg", mtf_persist_until_finished|mtf_culture_5|mtf_sit_killed, 0),
  ("killed_by_swadian", "killed_by_swadian.ogg", mtf_persist_until_finished|mtf_culture_1|mtf_sit_killed, 0),
#  ("killed_by_vaegir", "killed_by_vaegir.ogg", mtf_persist_until_finished|mtf_culture_2|mtf_sit_killed, 0),
  
  ("lords_hall_khergit", "lords_hall_khergit.ogg", mtf_culture_3|mtf_sit_travel, mtf_sit_town|mtf_sit_night|mtf_sit_tavern|mtf_culture_all),
  ("lords_hall_nord", "lords_hall_nord.ogg", mtf_sit_travel, mtf_sit_town|mtf_sit_night|mtf_sit_tavern),
  ("lords_hall_swadian", "lords_hall_swadian.ogg", mtf_sit_travel, mtf_sit_town|mtf_sit_night|mtf_sit_tavern),
  ("lords_hall_rhodok", "lords_hall_rhodok.ogg", mtf_sit_travel, mtf_sit_town|mtf_sit_night|mtf_sit_tavern),
  ("lords_hall_vaegir", "lords_hall_vaegir.ogg", mtf_sit_travel, mtf_sit_town|mtf_sit_night|mtf_sit_tavern),

  ("mounted_snow_terrain_calm", "mounted_snow_terrain_calm.ogg", mtf_sit_travel, mtf_sit_town|mtf_sit_night|mtf_sit_night|mtf_sit_tavern),
  ("neutral_infiltration", "neutral_infiltration.ogg", mtf_sit_town_infiltrate, 0),
  ("outdoor_beautiful_land", "outdoor_beautiful_land.ogg", mtf_sit_travel, mtf_sit_town|mtf_sit_night|mtf_sit_night|mtf_sit_tavern),
  ("retreat", "retreat.ogg", mtf_persist_until_finished|mtf_sit_killed, 0),

  ("seige_neutral", "seige_neutral.ogg", mtf_sit_siege, mtf_sit_fight|mtf_sit_multiplayer_fight|mtf_sit_ambushed),
  ("enter_the_juggernaut", "enter_the_juggernaut.ogg", mtf_sit_siege, mtf_sit_fight|mtf_sit_multiplayer_fight|mtf_sit_ambushed),  
  ("siege_attempt", "siege_attempt.ogg", mtf_sit_siege, mtf_sit_fight|mtf_sit_multiplayer_fight|mtf_sit_ambushed),  
  ("crazy_battle_music", "crazy_battle_music.ogg", mtf_sit_siege, mtf_sit_fight|mtf_sit_multiplayer_fight|mtf_sit_ambushed),
  
  ("tavern_1", "tavern_1.ogg", mtf_sit_tavern|mtf_sit_feast, 0),
  ("tavern_2", "tavern_2.ogg", mtf_sit_tavern|mtf_sit_feast, 0),

  ("town_neutral", "town_neutral.ogg", mtf_sit_town|mtf_sit_travel, mtf_sit_tavern|mtf_sit_night),
  ("town_khergit", "town_khergit.ogg", mtf_culture_3|mtf_sit_town|mtf_sit_travel, mtf_sit_tavern|mtf_sit_night|mtf_culture_all),
  ("town_nord", "town_nord.ogg", mtf_culture_4|mtf_sit_town|mtf_sit_travel, mtf_sit_tavern|mtf_sit_night|mtf_culture_all),
  ("town_rhodok", "town_rhodok.ogg", mtf_culture_5|mtf_sit_town|mtf_sit_travel, mtf_sit_tavern|mtf_sit_night|mtf_culture_all),
  ("town_swadian", "town_swadian.ogg", mtf_culture_1|mtf_sit_town|mtf_sit_travel, mtf_sit_tavern|mtf_sit_night|mtf_culture_all),
  ("town_vaegir", "town_vaegir.ogg", mtf_culture_2|mtf_sit_town|mtf_sit_travel, mtf_sit_tavern|mtf_sit_night|mtf_culture_all),

  ("travel_khergit", "travel_khergit.ogg", mtf_culture_3|mtf_sit_travel, mtf_sit_town|mtf_sit_tavern|mtf_sit_night|mtf_culture_all),
  ("travel_neutral", "travel_neutral.ogg", mtf_sit_travel, mtf_sit_town|mtf_sit_tavern|mtf_sit_night),
  ("travel_nord",    "travel_nord.ogg",    mtf_culture_4|mtf_sit_travel, mtf_sit_town|mtf_sit_tavern|mtf_sit_night|mtf_culture_all),
  ("travel_rhodok",  "travel_rhodok.ogg",  mtf_culture_5|mtf_sit_travel, mtf_sit_town|mtf_sit_tavern|mtf_sit_night|mtf_culture_all),
  ("travel_swadian", "travel_swadian.ogg", mtf_culture_1|mtf_sit_travel, mtf_sit_town|mtf_sit_tavern|mtf_sit_night|mtf_culture_all),
  ("travel_vaegir",  "travel_vaegir.ogg",  mtf_culture_2|mtf_sit_travel, mtf_sit_town|mtf_sit_tavern|mtf_sit_night|mtf_culture_all),
  ("travel_sarranid",  "middle_eastern_travel.ogg",  mtf_culture_6|mtf_sit_travel, mtf_sit_town|mtf_sit_tavern|mtf_sit_night|mtf_culture_all),

  ("uncertain_homestead", "uncertain_homestead.ogg", mtf_sit_travel, mtf_sit_town|mtf_sit_night|mtf_sit_tavern),
  ("hearth_and_brotherhood", "hearth_and_brotherhood.ogg", mtf_sit_travel, mtf_sit_town|mtf_sit_night|mtf_sit_tavern),
  ("tragic_village", "tragic_village.ogg", mtf_sit_travel, mtf_sit_town|mtf_sit_night|mtf_sit_tavern),

  ("victorious_evil", "victorious_evil.ogg", mtf_persist_until_finished, 0),
  ("victorious_neutral_1", "victorious_neutral_1.ogg", mtf_persist_until_finished|mtf_sit_victorious, 0),
  ("victorious_neutral_2", "victorious_neutral_2.ogg", mtf_persist_until_finished|mtf_sit_victorious, 0),
  ("victorious_neutral_3", "victorious_neutral_3.ogg", mtf_persist_until_finished|mtf_sit_victorious, 0),

  ("victorious_swadian", "victorious_swadian.ogg", mtf_persist_until_finished|mtf_culture_2|mtf_sit_victorious, 0),
  ("victorious_vaegir", "victorious_vaegir.ogg", mtf_persist_until_finished|mtf_culture_2|mtf_sit_victorious, 0),
  ("victorious_vaegir_2", "victorious_vaegir_2.ogg", mtf_persist_until_finished|mtf_culture_2|mtf_sit_victorious, 0),
  ("wedding", "wedding.ogg", mtf_persist_until_finished, 0),

  ("coronation", "coronation.ogg", mtf_persist_until_finished, 0),



  
]



from smart_module.module_system.module_animations import *
from smart_module.module_system.module_factions import *
from smart_module.module_system.module_game_menus import *
from smart_module.module_system.module_items import *
from smart_module.module_system.module_map_icons import *
from smart_module.module_system.module_meshes import *
from smart_module.module_system.module_mission_templates import *
from smart_module.module_system.module_music import *
from smart_module.module_system.module_particle_systems import *
from smart_module.module_system.module_parties import *
from smart_module.module_system.module_party_templates import *
from smart_module.module_system.module_presentations import *
from smart_module.module_system.module_quests import *
from smart_module.module_system.module_scene_props import *
from smart_module.module_system.module_scenes import *
from smart_module.module_system.module_scripts import *
from smart_module.module_system.module_skills import *
from smart_module.module_system.module_sounds import *
from smart_module.module_system.module_strings import *
from smart_module.module_system.module_tableau_materials import *
from smart_module.module_system.module_troops import *
from smart_module.module_system.process_common import *


def get_id_value(tag, identifier, tag_uses):
    tag_type = -1
    id_no = -1
    if (tag == "str"):
        id_no = find_object(strings, identifier)
        tag_type = tag_string
    elif (tag == "itm"):
        id_no = find_object(items, identifier)
        tag_type = tag_item
    elif (tag == "trp"):
        id_no = find_object(troops, identifier)
        tag_type = tag_troop
    elif (tag == "fac"):
        id_no = find_object(factions, identifier)
        tag_type = tag_faction
    elif (tag == "qst"):
        id_no = find_object(quests, identifier)
        tag_type = tag_quest
    elif (tag == "pt"):
        id_no = find_object(party_templates, identifier)
        tag_type = tag_party_tpl
    elif (tag == "p"):
        id_no = find_object(parties, identifier)
        tag_type = tag_party
    elif (tag == "scn"):
        id_no = find_object(scenes, identifier)
        tag_type = tag_scene
    elif (tag == "mt"):
        id_no = find_object(mission_templates, identifier)
        tag_type = tag_mission_tpl
    elif (tag == "mnu"):
        id_no = find_object(game_menus, identifier)
        tag_type = tag_menu
    elif (tag == "script"):
        id_no = find_object(scripts, identifier)
        tag_type = tag_script
    elif (tag == "psys"):
        id_no = find_object(particle_systems, identifier)
        tag_type = tag_particle_sys
    elif (tag == "spr"):
        id_no = find_object(scene_props, identifier)
        tag_type = tag_scene_prop
    elif (tag == "prsnt"):
        id_no = find_object(presentations, identifier)
        tag_type = tag_presentation
    elif (tag == "snd"):
        id_no = find_object(sounds, identifier)
        tag_type = tag_sound
    elif (tag == "icon"):
        id_no = find_object(map_icons, identifier)
        tag_type = tag_map_icon
    elif (tag == "skl"):
        id_no = find_object(skills, identifier)
        tag_type = tag_skill
    elif (tag == "track"):
        id_no = find_object(tracks, identifier)
        tag_type = tag_track
    elif (tag == "mesh"):
        id_no = find_object(meshes, identifier)
        tag_type = tag_mesh
    elif (tag == "anim"):
        id_no = find_object(animations, identifier)
        tag_type = tag_animation
    elif (tag == "tableau"):
        id_no = find_object(tableaus, identifier)
        tag_type = tag_tableau

    if (tag_type > -1 and id_no > -1):
        add_tag_use(tag_uses, tag_type, id_no)
    return (tag_type, id_no)


def get_identifier_value(str, tag_uses):
    underscore_pos = str.find( "_")
    result = -1
    if (underscore_pos > 0):
        tag_str = str[0:underscore_pos]
        id_str = str[underscore_pos + 1:len(str)]
        (tag_type, id_no) = get_id_value(tag_str, id_str, tag_uses)
        if (tag_type > 0):
            if (id_no < 0):
                print("Error: Unable to find object:" + str)
            else:
                result = id_no | (tag_type << op_num_value_bits)
        else:
            print("Error: Unrecognized tag:" + tag_str + "in object:" + str)
    else:
        print("Error: Invalid object:" + str + ".Variables should start with $ sign and references should start with a tag")
    return result


def load_quick_strings(export_dir):
    quick_strings = []
    try:
        file = open(export_dir + "quick_strings.txt", "r")
        str_list = file.readlines()
        file.close()
        for s in str_list:
            s = s.strip()
            if s:
                ssplit = s.split(' ')
                if len(ssplit) == 2:
                    quick_strings.append(ssplit)
    except:
        print("Creating new quick_strings.txt file...")
    return quick_strings


def save_quick_strings(export_dir, quick_strings):
    file = open(export_dir + "quick_strings.txt", "w")
    file.write("%d\n" % len(quick_strings))
    for i in range(len(quick_strings)):
        file.write("%s %s\n" % (quick_strings[i][0], replace_spaces(quick_strings[i][1])))
    file.close()


def load_variables(export_dir, variable_uses):
    variables = []
    try:
        file = open(export_dir + "variables.txt", "r")
        var_list = file.readlines()
        file.close()
        for v in var_list:
            vv = v.strip()
            if vv:
                variables.append(vv)
    except:
        print("variables.txt not found. Creating new variables.txt file")

    try:
        file = open(export_dir + "variable_uses.txt", "r")
        var_list = file.readlines()
        file.close()
        for v in var_list:
            vv = v.strip()
            if vv:
                variable_uses.append(int(vv))
    except:
        print("variable_uses.txt not found. Creating new variable_uses.txt file")

    return variables


def save_variables(export_dir, variables_list, variable_uses):
    file = open(export_dir + "variables.txt", "w")
    for i in range(len(variables_list)):
        file.write("%s\n" % variables_list[i])
    file.close()
    file = open(export_dir + "variable_uses.txt", "w")
    for i in range(len(variables_list)):
        file.write("%d\n" % variable_uses[i])
    file.close()


def ensure_tag_use(tag_uses, tag_no, object_no):
    if len(tag_uses[tag_no]) <= object_no:
        num_to_add = object_no + 1 - len(tag_uses[tag_no])
        for j in range(num_to_add):
            tag_uses[tag_no].append(0)


def add_tag_use(tag_uses, tag_no, object_no):
    # TODO: Uncomment to make build_module_check_tags work
    #  ensure_tag_use(tag_uses, tag_no, object_no)
    #  tag_uses[tag_no][object_no] = tag_uses[tag_no][object_no] + 1
    pass


def load_tag_uses(export_dir):
    tag_uses = []
    for i in range(tags_end):
        sub_tag_uses = []
        tag_uses.append(sub_tag_uses)

    try:
        file = open(export_dir + "tag_uses.txt", "r")
        var_list = file.readlines()
        file.close()
        for v in var_list:
            vv = v.strip().split(';')
            if vv:
                for v2 in vv:
                    vvv = v2.split(' ')
                    if len(vvv) >= 3:
                        ensure_tag_use(tag_uses, int(vvv[0]), int(vvv[1]))
                        tag_uses[int(vvv[0])][int(vvv[1])] = int(vvv[2])
    except:
        print("Creating new tag_uses.txt file...")
    return tag_uses


def save_tag_uses(export_dir, tag_uses):
    file = open(export_dir + "tag_uses.txt", "w")
    for i in range(len(tag_uses)):
        for j in range(len(tag_uses[i])):
            file.write("%d %d %d;" % (i, j, tag_uses[i][j]))
        file.write("\n")
    file.close()


def add_cookie(cookies_list, cookie_string):
    found = 0
    result = -1
    for i_t in range(len(cookies_list)):
        if cookie_string == cookies_list[i_t]:
            found = 1
            result = i_t
            break
    if not found:
        cookies_list.append(cookie_string)
        result = len(cookies_list) - 1
    return result


def get_cookie(cookies_list, cookie_string):
    found = 0
    result = -1
    for i_t in range(len(cookies_list)):
        if cookie_string == cookies_list[i_t]:
            found = 1
            result = i_t
            break
    if not found:
        print("ERROR: input token not found:" + cookie_string)
        cookies_list.append(cookie_string)
        result = len(cookies_list) - 1
    return result


def check_varible_not_defined(variable_string, variables_list):
    for i_t in range(len(variables_list)):
        if variable_string == variables_list[i_t]:
            print("WARNING: Variable name used for both local and global contexts:" + variable_string)
            break


# def add_get_variable(variable_string,variables_list):
#  found = 0
#  result = -1
#  for i_t in range(len(variables_list)):
#    if variable_string == variables_list[i_t]:
#      found = 1
#      result = i_t
#      break
#  if not found:
#    variables_list.append(variable_string)
#    result = len(variables_list) - 1
#  return result

def add_variable(variable_string, variables_list, variable_uses):
    found = 0
    for i_t in range(len(variables_list)):
        if variable_string == variables_list[i_t]:
            found = 1
            variable_uses[i_t] = variable_uses[i_t] - 1
            break
    if not found:
        variables_list.append(variable_string)
        variable_uses.append(-1)


def get_variable(variable_string, variables_list, variable_uses):
    found = 0
    result = -1
    var_string = variable_string[1:]
    for i_t in range(len(variables_list)):
        if var_string == variables_list[i_t]:
            found = 1
            result = i_t
            variable_uses[result] = variable_uses[result] + 1
            break
    if not found:
        if (variable_string[0] == '$'):
            variables_list.append(variable_string)
            variable_uses.append(0)
            result = len(variables_list) - 1
            print("WARNING: Usage of unassigned global variable: " + variable_string)
        else:
            print("ERROR: Usage of unassigned local variable: " + variable_string)
    return result


def is_lhs_operation(op_code):
    found = 0
    if op_code in lhs_operations:
        return 1
    return 0


def is_lhs_operation_for_global_vars(op_code):
    found = 0
    if op_code in lhs_operations:
        return 1
    if op_code in global_lhs_operations:
        return 1
    return 0


def is_can_fail_operation(op_code):
    found = 0
    if op_code in can_fail_operations:
        return 1
    return 0


def search_quick_string_keys(key, quick_strings):
    index = -1
    for i in range(len(quick_strings)):
        if quick_strings[i][0] == key:
            index = i
    return index


def insert_quick_string_with_auto_id(sentence, quick_strings):
    index = 0
    text = convert_to_identifier_with_no_lowercase(sentence)
    sentence = replace_spaces(sentence)
    done = 0
    i = 20
    lt = len(text)
    if (i > lt):
        i = lt
    auto_id = "qstr_" + text[0:i]
    done = 0
    index = search_quick_string_keys(auto_id, quick_strings)
    if index >= 0 and (quick_strings[index][1] == sentence):
        done = 1
    while (i <= lt) and not done:
        auto_id = "qstr_" + text[0:i]
        index = search_quick_string_keys(auto_id, quick_strings)
        if index >= 0:
            if quick_strings[index][1] == sentence:
                done = 1
            else:
                i += 1
        else:
            done = 1
            index = len(quick_strings)
            quick_strings.append([auto_id, sentence])
    if not done:
        number = 1
        new_auto_id = auto_id + str(number)
        while quick_strings.has_key(new_auto_id):
            number += 1
            new_auto_id = auto_id + str(number)
        auto_id = new_auto_id
        index = len(quick_strings)
        quick_strings.append([auto_id, sentence])
    return index


def process_param(param, global_vars_list, global_var_uses, local_vars_list, local_var_uses, tag_uses, quick_strings):
    result = 0
    if (type(param) == str):
        if (param[0] == '$'):
            check_varible_not_defined(param[1:], local_vars_list)
            result = get_variable(param, global_vars_list, global_var_uses)
            result |= opmask_variable
        elif (param[0] == ':'):
            check_varible_not_defined(param[1:], global_vars_list)
            result = get_variable(param, local_vars_list, local_var_uses)
            result |= opmask_local_variable
        elif (param[0] == '@'):
            result = insert_quick_string_with_auto_id(param[1:], quick_strings)
            result |= opmask_quick_string
        else:
            result = get_identifier_value(param.lower(), tag_uses)
            if (result < 0):
                print("ERROR: Illegal Identifier:" + param)
    else:
        result = param
    return result


def save_statement(ofile, opcode, no_variables, statement, variable_list, variable_uses, local_vars_list,
                   local_var_uses, tag_uses, quick_strings):
    if no_variables == 0:
        lenstatement = len(statement) - 1
        if (is_lhs_operation(opcode) == 1):
            if (lenstatement > 0):
                param = statement[1]
                if (type(param) == str):
                    if (param[0] == ':'):
                        add_variable(param[1:], local_vars_list, local_var_uses)
    else:
        lenstatement = 0
    ofile.write("%d %d " % (opcode, lenstatement))
    for i in range(lenstatement):
        operand = process_param(statement[i + 1], variable_list, variable_uses, local_vars_list, local_var_uses,
                                tag_uses, quick_strings)
        ofile.write("%d " % operand)


def compile_global_vars_in_statement(statement, variable_list, variable_uses):
    opcode = 0
    if ((type(statement) != list) and (type(statement) != tuple)):
        opcode = statement
    else:
        opcode = statement[0]
        if (is_lhs_operation_for_global_vars(opcode) == 1):
            if (len(statement) > 1):
                param = statement[1]
                if (type(param) == str):
                    if (statement[1][0] == '$'):
                        add_variable(statement[1][1:], variable_list, variable_uses)


def save_statement_block(ofile, statement_name, can_fail_statement, statement_block, variable_list, variable_uses,
                         tag_uses, quick_strings):
    local_vars = []
    local_var_uses = []
    ofile.write(" %d " % (len(statement_block)))
    store_script_param_1_uses = 0
    store_script_param_2_uses = 0
    current_depth = 0
    can_fail = 0
    for i in range(len(statement_block)):
        statement = statement_block[i]
        if ((type(statement) != list) and (type(statement) != tuple)):
            opcode = statement
            no_variables = 1
        else:
            opcode = statement[0]
            no_variables = 0
        if (opcode in [try_begin,
                       try_for_range,
                       try_for_range_backwards,
                       try_for_parties,
                       try_for_agents]):
            current_depth = current_depth + 1
        elif (opcode == try_end):
            current_depth = current_depth - 1
        elif (opcode == store_script_param_1 or (opcode == store_script_param and statement[2] == 1)):
            store_script_param_1_uses = store_script_param_1_uses + 1
        elif (opcode == store_script_param_2 or (opcode == store_script_param and statement[2] == 2)):
            store_script_param_2_uses = store_script_param_2_uses + 1
        elif (can_fail_statement == 0 and current_depth == 0
              and (is_can_fail_operation(opcode)
                   or ((opcode == call_script) and (statement[1].startswith("cf_", 7))))
              and (not statement_name.startswith("cf_"))):
            print("WARNING: Script can fail at operation #" + str(i) + ". Use cf_ at the beginning of its name: " + statement_name)
        save_statement(ofile, opcode, no_variables, statement, variable_list, variable_uses, local_vars, local_var_uses,
                       tag_uses, quick_strings)
    if (store_script_param_1_uses > 1):
        print("WARNING: store_script_param_1 is used more than once:" + statement_name)
    if (store_script_param_2_uses > 1):
        print("WARNING: store_script_param_2 is used more than once:" + statement_name)
    i = 0
    while (i < len(local_vars)):
        if (local_var_uses[i] == 0 and not (local_vars[i].startswith("unused"))):
            print("WARNING: Local variable never used: " + local_vars[i] + ", at: " + str(statement_name))
        i = i + 1
    if (len(local_vars) > 128):
        print("WARNING: Script uses more than 128 local wariables: " + str(statement_name) + "variables count:" + str(len(local_vars)))


def compile_global_vars(statement_block, variable_list, variable_uses):
    for statement in statement_block:
        compile_global_vars_in_statement(statement, variable_list, variable_uses)


def save_simple_triggers(ofile, triggers, variable_list, variable_uses, tag_uses, quick_strings):
    ofile.write("%d\n" % len(triggers))
    for trigger in triggers:
        ofile.write("%f " % (trigger[0]))
        save_statement_block(ofile, 0, 1, trigger[1], variable_list, variable_uses, tag_uses, quick_strings)
        ofile.write("\n")
    ofile.write("\n")



from smart_module.module_system.header_common import *

###################################################
# header_scenes.py
# This file contains declarations for scenes
# DO NOT EDIT THIS FILE!
###################################################

scene_name_pos = 0
passages_pos = 8

#flags
sf_indoors           = 0x00000001   #The scene shouldn't have a skybox and lighting by sun.
sf_force_skybox      = 0x00000002   #Force adding a skybox even if indoors flag is set.
sf_generate          = 0x00000100   #Generate terrain by terran-generator
sf_randomize         = 0x00000200   #Randomize terrain generator key
sf_auto_entry_points = 0x00000400   #Automatically create entry points
sf_no_horses         = 0x00000800   #Horses are not avaible
sf_muddy_water       = 0x00001000   #Changes the shader of the river mesh





from smart_module.module_system.module_items import items
from smart_module.module_system.module_scripts import *


###########################
##
## 村庄基础业务
## 1.种植
## 2.畜牧
## 3.矿产
## 4.手工
##
###########################



from smart_module.module_system.module_troops import *
from smart_module.module_system.header_dialogs import *
from smart_module.smart_core.base.meta_command.meta_command_processor import *
from smart_module.smart_core.smart_module_slot import smartModuleSlotManager

## 收获时间
## 一块地 一个月收入不能超过500，每个星期不超过125（否则游戏就是作弊）
##villageBasicBusinessUpdateTime = 24 * 7 * 4
villageBasicBusinessUpdateTime = 24 * 7 * 4

## 配置
villageBasicBusinessConfig = {
    ## 每一级统率增加多少生产率
    "perLeadershipProductivityIncrease":0.1,
    ## 每一级统率可以多管理的土地数量
    "perLeadershipManageFieldCount":10,
    ## 每一块土地需要租金
    "perFieldCostAmount":1000,
    ## 村庄最大生产能力（0：该村庄无法生产此类商品 100：该村庄非常善于生产此类产品）
    "perVillageMaxProductivityAbility":100,
    ## 【收入】交易技能加成比例：交易技能最大为10，如果设置为10，加成就是总收入的一倍，如果是100，加成就是总收入的1/10
    "tradeSkillRate":100,
    ## 【产量】管理技能加成比例：交易技能最大为10，如果设置为10，加成就是总产量的一倍，如果是100，加成就是总产量的1/10
    "leadershipSkillRate":30,
    ## 工作间
    "workshopList":[
        {
            "name":"plant",
            "nameCns":"种 植 业",
            "prodectList":[
                {
                    "name":"grain",
                    "nameCns":"小 麦",
                    "productionQuantity":30,
                    "itemNo":"itm_grain",
                    "overhead":600,
                    "desc":"利润 = 30 * 30 - 600 = 300"
                },
                {
                    "name":"raw_grapes",
                    "nameCns":"葡 萄",
                    "productionQuantity":10,
                    "itemNo":"itm_raw_grapes",
                    "overhead":500,
                    "desc":"利润 = 10 * 75 - 500 = 250"
                },
                {
                    "name":"raw_olives",
                    "nameCns":"橄 榄",
                    "productionQuantity":15,
                    "itemNo":"itm_raw_olives",
                    "overhead":1150,
                    "desc":"利润 = 15 * 100 - 1150 = 350"
                },
                {
                    "name":"apples",
                    "nameCns":"苹 果",
                    "productionQuantity":15,
                    "itemNo":"itm_apples",
                    "overhead":400,
                    "desc":"利润 = 15 * 44 - 400 = 260"
                }
            ]
        },
        {
            "name":"husbandry",
            "nameCns":"畜 牧 业",
            "prodectList":[
                {
                    "name":"tutorial_saddle_horse",
                    "nameCns":"驮 马",
                    "productionQuantity":10,
                    "itemNo":"itm_sumpter_horse",
                    "overhead":1000,
                    "desc":"利润 = 10 * 134 - 1000 = 340"
                },
                {
                    "name":"steppe_horse",
                    "nameCns":"草 原 马",
                    "productionQuantity":10,
                    "itemNo":"itm_steppe_horse",
                    "overhead":1800,
                    "desc":"利润 = 10 * 192 - 1800 = 120"
                },
                {
                    "name":"hunter",
                    "nameCns":"猎 马",
                    "productionQuantity":5,
                    "itemNo":"itm_hunter",
                    "overhead":3800,
                    "desc":"利润 = 5 * 810 - 3800 = 250"
                },
            ]
        },
        {
            "name":"minerals",
            "nameCns":"矿 产 业",
            "prodectList":[
                {
                    "name":"iron",
                    "nameCns":"生 铁",
                    "productionQuantity":30,
                    "itemNo":"itm_iron",
                    "overhead":7500,
                    "desc":"利润 = 30 * 264 - 7500 = 420"
                },
                {
                    "name":"raw_dyes",
                    "nameCns":"染 料",
                    "productionQuantity":30,
                    "itemNo":"itm_raw_dyes",
                    "overhead":5200,
                    "desc":"利润 = 30 * 200 - 5200 = 800"
                },
            ]
        },
        {
            "name":"manufacture",
            "nameCns":"制 造 业",
            "prodectList":[
                {
                    "name":"tools",
                    "nameCns":"工 具",
                    "productionQuantity":20,
                    "itemNo":"itm_tools",
                    "overhead":7500,
                    "desc":"利润 = 20 * 410 - 7500 = =700"
                },
                {
                    "name":"leatherwork",
                    "nameCns":"皮 革 制 品",
                    "productionQuantity":30,
                    "itemNo":"itm_leatherwork",
                    "overhead":5800,
                    "desc":"利润 = 30 * 220 - 5800 = 800"
                },
                {
                    "name":"bread",
                    "nameCns":"面 包",
                    "productionQuantity":30,
                    "itemNo":"itm_bread",
                    "overhead":1200,
                    "desc":"利润 = 30 * 50 - 1200 = 300"
                },
            ]
        },
    ],
}

## 解析配置
perLeadershipProductivityIncrease = villageBasicBusinessConfig["perLeadershipProductivityIncrease"]
perLeadershipManageFieldCount = villageBasicBusinessConfig["perLeadershipManageFieldCount"]
perFieldCostAmount = villageBasicBusinessConfig["perFieldCostAmount"]
perVillageMaxProductivityAbility = villageBasicBusinessConfig["perVillageMaxProductivityAbility"]
workshopList = villageBasicBusinessConfig["workshopList"]

workShopCount = len(workshopList)

leadershipSkillRate = villageBasicBusinessConfig["leadershipSkillRate"]
tradeSkillRate = villageBasicBusinessConfig["tradeSkillRate"]

workShopListArray = "trp_workshop_list"



productListArray = "trp_product_list"



## 每个工作间属性个数
workShopPropertyCount = 5
## 每个商品属性个数
productPropertyCount = 5

## 1.初始化数据
## 2.

## 初始化商店数据(解析配置文件并生成游戏能够识别的数据格式)
##  工作间：1.名称 2.产品名称开始编号 3.产品名称结束编号 4.产品数量 5.每一个工作间第一个产品在产品列表中开始索引
##  产品：1.名称 2.产量 3.物品编号 4.日常开销
##
##
def initWorkShop():
    global workShopPropertyCount,productPropertyCount

    workShopDatas = []
    productDatas = []
    shopNameStrDatas = []
    productNameStrDatas = []
    productCount = 0;
    productStartIndex = 0;
    ## 数组中第0位保存商店个数
    workShopDatas.append(workShopCount)
    ## 占用第一位，计算后再保存一次
    productDatas.append(0)
    for workShop in workshopList:
        ## 生成商店字符串数据
        shopCode = workShop["name"]
        shopNameCode = "shop_name_{}".format(shopCode)
        shopName = workShop["nameCns"]
        shopNameStrDatas.append((shopNameCode,shopName))
        ## 保存商店字符串数据到列表
        workShopDatas.append("str_" + shopNameCode)
        ## 统计商品数量
        prodectList = workShop["prodectList"]
        productCount = productCount + len(prodectList)
        shopNameStartIndex = ""
        shopNameEndIndex = ""
        for i in range(len(prodectList)):
            product = prodectList[i]
            productCode = product["name"]
            productionQuantity = product["productionQuantity"]
            itemNo = product["itemNo"]
            overhead = product["overhead"]
            ## 生成产品字符串数据
            producNameCode = "product_name_{}_{}".format(shopCode,productCode)
            productName = product["nameCns"]
            ## 保存商品字符串数据
            productNameStrDatas.append((producNameCode,productName))
            ## 计算商品名称开始位置
            if i == 0:
                shopNameStartIndex = producNameCode
            elif i == len(prodectList) - 1:
                productEndCode = "product_name_{}_end".format(shopCode)
                productNameStrDatas.append((productEndCode,"product name {} end".format(shopCode)))
                shopNameEndIndex = productEndCode

            ## 商品名称 1
            productDatas.append("str_" + producNameCode)
            ## 每块土地产量 2
            productDatas.append(productionQuantity)
            ## 产品编号 3
            productDatas.append(itemNo)
            ## itm_开头
            itemKey = itemNo[4:]
            itemPrice = None
            ## 产品价格 4
            for item in items:
                if item[0] == itemKey:
                    itemPrice = item[5]
                    break;
            if itemPrice == None:
                raise RuntimeError("未找到物品：" + itemNo)
            productDatas.append(itemPrice)
            ## 开销：5
            productDatas.append(overhead)



        workShopDatas.append("str_" + shopNameStartIndex)
        workShopDatas.append("str_" + shopNameEndIndex)
        workShopDatas.append(len(prodectList))
        workShopDatas.append(productStartIndex)
        productStartIndex = productStartIndex + len(prodectList)

    productDatas[0] = productCount

    ## 生成字符串信息
    strs = shopNameStrDatas + productNameStrDatas
    ## 生成初始化指令
    commands = []

    for i in range(len(workShopDatas)):
        shopData = workShopDatas[i]
        commands.append((troop_set_slot, workShopListArray, i, shopData))

    for i in range(len(productDatas)):
        productData = productDatas[i]
        commands.append((troop_set_slot, productListArray, i, productData))
    return strs,commands


basicBusinessData = initWorkShop()

basicBusinessInitStrs = basicBusinessData[0]
basicBusinessInitCommands = basicBusinessData[1]

## troop slot

## 管理员管理的土地最终的老板
slot_troop_workshop_boss = smartModuleSlotManager.getTroopSlotNo("slot_troop_workshop_boss")
## 管理员管理的土地数量
slot_troop_workshop_count = smartModuleSlotManager.getTroopSlotNo("slot_troop_workshop_count")
## 管理员管理的商品类型(item_id)
slot_troop_workshop_product_type = smartModuleSlotManager.getTroopSlotNo("slot_troop_workshop_product_type")
## 产品基础价格
slot_troop_workshop_product_price = smartModuleSlotManager.getTroopSlotNo("slot_troop_workshop_product_price")
## 管理员管理的商品每块地基础产量
slot_troop_workshop_product_quantity = smartModuleSlotManager.getTroopSlotNo("slot_troop_workshop_product_quantity")
## 管理员管理的商品每块地日常开销（按季来算）
slot_troop_workshop_product_overhead = smartModuleSlotManager.getTroopSlotNo("slot_troop_workshop_product_overhead")
## 管理员管理的商品的存储策略
slot_troop_workshop_storage_strategy = smartModuleSlotManager.getTroopSlotNo("slot_troop_workshop_storage_strategy")
## 管理员管理的所有土地下一季的产量（理论值）
slot_troop_workshop_product_quarterly_output  = smartModuleSlotManager.getTroopSlotNo("slot_troop_workshop_product_quarterly_output")



## 【存储策略类型】
## 全部保存（超出出售）
sltwss_sell_remainder = 0
## 全部出售
sltwss_sell_alll = 1
## 全部保留（超出丢弃）
sltwss_keep_alll = 2


## slot_troop_occupation 的值
## 工作间管理员
slto_workshop_admin = 100



villageBasicBusiness = {
    "name":"VillageBasicBusiness",
    "enable":True,
    "version":"v2.0.0",
    "desc":'''
        村庄基础业务,买地，开工作间，选择产品类型，每月获得收益
        1.种植
        2.畜牧
        3.矿产
        4.手工
    ''',
    "commands":{
        "AddDialogForWorkshopAdmin":{
            "target":"dialogs",
            "selector":"companion_rehire&1:Welcome_back__my_friend%^&3:companion_recruit_signup_confirm&4",
            "processor":PrependProcessor,
            "desc": "为管理员新增一个对话"
        }
    },
    "actions":[
        ("GameInitScript",[
            ##(display_message,"@base basic business data init"),
            (call_script,"script_init_village_base_basic_business_data"),
        ]),
        ("Append@scripts",[
            ("init_village_base_basic_business_data",basicBusinessInitCommands),
            ("get_current_center_workshop_admin",[
                (store_script_param_1,":center"),
                ## 避免上一次计算结果影响本次计算
                (assign,reg0,-1),
                (try_for_range, ":companion", companions_begin, companions_end),
                    ## npc是管理员
                    (troop_slot_eq,":companion",slot_troop_occupation,slto_workshop_admin),
                    ## npc所在地是当前地
                    (troop_slot_eq,":companion",slot_troop_cur_center,":center"),
                    (assign,reg0,":companion"),
                    ## 跳转循环
                    (assign,":companion",companions_end),
                (try_end),
            ]),
        ]),
        ("Append@simple_triggers",[
            ## 增加税收
            (villageBasicBusinessUpdateTime, [
                (display_message, "@It's harvest time again"),
                (assign,":allTotalAmount",0),
                (try_for_range, ":companion", companions_begin, companions_end),
                    ## npc是管理员
                    (troop_slot_eq, ":companion", slot_troop_occupation, slto_workshop_admin),
                    ## npc所在地
                    (troop_get_slot,":center", ":companion", slot_troop_cur_center),
                    ## 管理员名称
                    (str_store_troop_name,s0,":companion"),
                    ## 管理员所在地
                    (str_store_party_name_link,s1,":center"),
                    ## 产品类型
                    (troop_get_slot,":productType",":companion",slot_troop_workshop_product_type),
                    (try_begin),
                        ## 没有选择商品类型，无法获得产值
                        (le,":productType",0),
                        (display_message,"@{s0} {s1} Please set product type!"),
                    (else_try),
                        ## 【获得产品】这一季产量
                        (troop_get_slot,":quarterlyOutput",":companion",slot_troop_workshop_product_quarterly_output),
                        ## 土地维护花费（保存策略）
                        (troop_get_slot,":storageStrategy",":companion",slot_troop_workshop_storage_strategy),
                        ## 获得管理员仓库容量
                        (call_script,"script_get_troop_inventory_free_amount",":companion"),
                        (assign,":storageMax",reg0),
                        ## 获得单个产品价格
                        (troop_get_slot,":productPrice",":companion",slot_troop_workshop_product_price),
                        ## 获得交易能力
                        (store_skill_level, ":tradeLevel", skl_trade, ":companion"),

                        ## 即将卖出的数量
                        (assign,":sellProductCount",0),
                        ## 保存到仓库中的数量（没有卖出的数量）
                        (assign,":keepProductCount",0),
                        ## 超出仓库的数量
                        (assign,":moreProductCount",0),
                        ## 是否出售超出仓库的数量的产品： 0  出售  1.不出售
                        ## 只是用于判定是否出售，以便显示消息
                        (assign,":isSellMore",0),
                        (try_begin),
                            ## 出售全部商品
                            (eq,":storageStrategy",sltwss_sell_alll),
                            (assign,":sellProductCount",":quarterlyOutput"),
                        (else_try),
                            (eq,":storageStrategy",sltwss_keep_alll),
                            (try_begin),
                                ## 如果产量超出仓库的数量
                                (gt,":quarterlyOutput",":storageMax"),
                                (assign,":keepProductCount",":storageMax"),
                                (store_sub,":moreProductCount",":quarterlyOutput",":storageMax"),
                                ## 剩余的产品用于和管理员增加关系(未实现)
                                (assign,":isSellMore",1),
                            (else_try),
                                ## 如果小于仓库保存的数量
                                (assign,":keepProductCount",":quarterlyOutput"),
                            (try_end),
                        (else_try),
                            ## 超出部分卖出
                            (try_begin),
                                ## 如果产量超出仓库的数量
                                (gt,":quarterlyOutput",":storageMax"),
                                (store_sub,":moreProductCount",":quarterlyOutput",":storageMax"),
                                (assign,":sellProductCount",":moreProductCount"),
                                (assign,":keepProductCount",":storageMax"),
                            (else_try),
                                ## 如果小于仓库保存的数量
                                (assign,":keepProductCount",":quarterlyOutput"),
                            (try_end),
                        (try_end),

                        ## 土地数量
                        (troop_get_slot, ":workshopCount", ":companion", slot_troop_workshop_count),

                        ## 产品总价 = 出售数量 * 单价
                        (store_mul,":totalAmount",":sellProductCount",":productPrice"),
                        ## 额外加成 = 产品总价 * 交易技能(最大为10) / 10
                        (store_mul,":plusAmount",":tradeLevel",":totalAmount"),
                        (val_div,":plusAmount",tradeSkillRate),
                        ## 日常开销 = 每块地开销 * 土地数量
                        (troop_get_slot,":quarterlyOverhead",":companion",slot_troop_workshop_product_overhead),
                        (assign,reg10,":quarterlyOverhead"),
                        ##(display_message,"@quarterlyOverhead:{reg10}"),
                        (store_mul,":totalOverhead",":quarterlyOverhead",":workshopCount"),
                        ## 实际收入 = 基础收入 + 管理员加成 - 维护开销费用
                        (store_add,":amountOfCenter",":totalAmount",":plusAmount"),
                        (val_sub,":amountOfCenter",":totalOverhead"),
                        ## 产品名称
                        (str_store_item_name,s2,":productType"),
                        ## 产量
                        (assign,reg0,":quarterlyOutput"),
                        ## 出售数量
                        (assign,reg1,":sellProductCount"),
                        ## 保存到仓库的数量
                        (assign,reg2,":keepProductCount"),
                        (assign,reg20,":workshopCount"),
                        (assign,reg21,":tradeLevel"),
                        (display_message,"@{s0} {s1} products {reg0}{s2} (field {reg20} sell {reg1} keep {reg2})"),
                        ## 将物品保存到仓库
                        (troop_add_items,":companion",":productType",":keepProductCount"),

                        (assign,reg3,":amountOfCenter"),
                        (assign,reg4,":totalAmount"),
                        (assign,reg5,":plusAmount"),
                        (assign,reg6,":totalOverhead"),
                        (display_message,"@{s0} {s1} sell amount {reg3} (field:{reg20} base sell:{reg4},trade:{reg21} admin plus:{reg5} cost:{reg6})"),
                        ## 统计汇总
                        (val_add,":allTotalAmount",":amountOfCenter"),
                        ## 【计算下一季产品】
                        ## 产品基础数量
                        (troop_get_slot,":productQuantity",":companion",slot_troop_workshop_product_quantity),
                        ## 土地维护花费（每块地）
                        (troop_get_slot,":productOverhead",":companion",slot_troop_workshop_product_overhead),
                        ## 获得管理能力
                        (store_skill_level, ":leadershipLevel", skl_leadership, ":companion"),
                        ## 产量 = 每块产量 * 土地数量
                        ## 管理加成 = 产量 * 管理技能 / 10
                        (store_mul,":baseOutput",":productQuantity",":workshopCount"),
                        (store_mul,":plusOutput",":baseOutput",":leadershipLevel"),
                        (val_div,":plusOutput",leadershipSkillRate),
                        (store_add,":quarterlyOutput",":baseOutput",":plusOutput"),
                        (assign,reg6,":quarterlyOutput"),
                        (assign,reg30,":plusOutput"),
                        (assign,reg31,":leadershipLevel"),
                        (display_message,"@next quarterly output:{reg6} leadership:{reg31} plus:{reg30}"),
                        (troop_set_slot,":companion",slot_troop_workshop_product_quarterly_output,":quarterlyOutput"),
                    (try_end),
                (try_end),
                (try_begin),
                    (gt,":allTotalAmount",0),
                    (call_script, "script_troop_add_gold", "trp_player", ":allTotalAmount"),
                (else_try),
                    (val_abs,":allTotalAmount"),
                    (troop_remove_gold, "trp_player", ":allTotalAmount"),
                (try_end),
            ]),
        ]),
        ("AppendCustomArrayTroop",[
            ## 用于保存工作间属性列表
            ["workshop_list", "workshop list", "workshop list", tf_hero, no_scene, reserved, fac_commoners, [],
             def_attrib, 0, knows_common | knows_inventory_management_10, 0],
            ## 用于保存商品属性列表
            ["product_list", "workshop list", "workshop list", tf_hero, no_scene, reserved, fac_commoners, [],
             def_attrib, 0, knows_common | knows_inventory_management_10, 0],
        ]),
        ("Append@strings",basicBusinessInitStrs),
        ("AddDialogForVillage",[
            [anyone | plyr, "village_elder_talk", [], "I want to rent a piece of land and build a workshop.",
             "talk_elder_workshop_rent_land", []],

            ## 管理员选择
            [anyone, "talk_elder_workshop_rent_land", [
                (call_script,"script_get_current_center_workshop_admin","$current_town"),
                ## 如果本地没有管理员
                (lt,reg0,0)
            ], "Who will manage it?",
             "talk_elder_workshop_admin_list_choice", []],

            [anyone, "talk_elder_workshop_rent_land", [
                (call_script,"script_get_current_center_workshop_admin","$current_town"),
                ## 如果本地没有管理员
                (gt,reg0,0)
            ], "There's already an administrator. He's right there!",
             "close_window", []],

            [anyone | plyr | repeat_for_troops, "talk_elder_workshop_admin_list_choice", [
                (store_repeat_object, ":currentTroop"),
                (is_between, ":currentTroop", companions_begin, companions_end),
                (troop_slot_eq, ":currentTroop", slot_troop_occupation, slto_player_companion),
                (store_skill_level, ":leadershipLevel", skl_leadership, ":currentTroop"),
                ## 统御技能要大于0，才可以被任命为管理员
                (gt, ":leadershipLevel", 0),
                (assign, reg1, ":leadershipLevel"),
                (str_store_troop_name, s1, ":currentTroop"),
            ], "{s1}(leadershipLevel:{reg1})", "talk_elder_workshop_rent_land_count", [
                 (store_repeat_object, ":troop"),
                 (assign, "$current_land_admin", ":troop"),
             ]],
            ## 结束管理员选择
            [anyone | plyr, "talk_elder_workshop_admin_list_choice", [], "I don't have the right person.",
             "close_window", []],

            [anyone, "talk_elder_workshop_rent_land_count", [], "How many pieces of land do you want to rent?",
             "talk_elder_workshop_rent_land_count_choice", []],
            [anyone | plyr | repeat_for_100, "talk_elder_workshop_rent_land_count_choice", [
                (store_repeat_object, ":index"),
                (is_between, ":index", 0, 10),
                ## 获得土地数量
                (store_mul, ":landCount", ":index", 1),
                (val_add, ":landCount", 1),
                (assign, reg1, ":landCount"),
                (store_mul, ":landAmount", ":landCount", perFieldCostAmount),
                (assign, reg2, ":landAmount"),
            ], "{reg1}(${reg2})", "close_window", [
                 (store_repeat_object, ":index"),
                 (store_mul, ":landCount", ":index", 1),
                 (val_add, ":landCount", 1),
                 (assign, reg3, ":landCount"),
                 ## 设置管理员的老板为玩家
                 (troop_set_slot, "$current_land_admin", slot_troop_workshop_boss, "trp_player"),
                 ## 设置同伴的职业为工作间管理员
                 (troop_set_slot, "$current_land_admin", slot_troop_occupation, slto_workshop_admin),
                 ## 设置管理员接手的工作间数量
                 (troop_set_slot, "$current_land_admin", slot_troop_workshop_count, ":landCount"),
                 ## 将管理员放置到该村庄
                 (store_current_scene, ":cur_scene"),
                 ##(add_troop_to_site,"$current_land_admin",":cur_scene",35),
                 # (set_visitor,35, "$current_town"),
                 ## (add_visitors_to_current_scene, 35, "$current_land_admin", 1, 0, 0),
                 (modify_visitors_at_site, ":cur_scene"),
                 (set_visitor, 32, "$current_land_admin"),
                 ## 设置管理员所在地方
                 (troop_set_slot, "$current_land_admin", slot_troop_cur_center, "$current_town"),
                 ## 同伴离队
                 (party_remove_members, "p_main_party", "$current_land_admin", 1),

                ## 存储策略（默认全部出售）
                (troop_set_slot, "$current_land_admin", slot_troop_workshop_storage_strategy, sltwss_sell_alll),

                 (store_mul, ":landAmount", ":landCount", perFieldCostAmount),
                 (assign, reg4, ":landAmount"),
                 (troop_remove_gold, "trp_player", ":landAmount"),
                 (display_message, "@cost {reg4} for {reg3} of lands."),
             ]],

            # ## 结束管理员选择
            [anyone | plyr, "talk_elder_workshop_rent_land_count_choice", [], "I can't afford it.", "close_window", []],

        ]),
        ("AddDialogForWorkshopAdmin",[
            ## 管理员对话
            [anyone|plyr, "companion_rehire", [
                (troop_slot_eq, "$g_talk_troop", slot_troop_occupation, slto_workshop_admin),
            ], "Let's talk about the workshop.", "workshop_admin_start", []],

            [anyone, "workshop_admin_start", [], "What would you like to do?", "workshop_admin_todo_list", []],

            ## 结束工作间事项对话
            [anyone|plyr, "companion_rehire", [
                (troop_slot_eq, "$g_talk_troop", slot_troop_occupation, slto_workshop_admin),
            ], "don't worry.", "close_window", []],

            ## 事项列表1：作间类型
            [anyone|plyr, "workshop_admin_todo_list", [], "Let's talk about the workshop.", "workshop_admin_worksho_type_start", []],
            ## 工作间类型选择
            [anyone, "workshop_admin_worksho_type_start", [], "What kind of workshop are we going to build?", "workshop_admin_land_type", []],
            [anyone | plyr | repeat_for_100, "workshop_admin_land_type", [
                (store_repeat_object, ":workShopIndex"),
                (troop_get_slot, ":workShopMax", workShopListArray, 0),
                (is_between, ":workShopIndex", 0, ":workShopMax"),
                ## 获得工作间名称
                (store_mul, ":workShopNameIndex", ":workShopIndex", workShopPropertyCount),
                (val_add, ":workShopNameIndex", 1),
                (troop_get_slot, ":workShopName", workShopListArray, ":workShopNameIndex"),
                (str_store_string, s1, ":workShopName"),
            ], "{s1}", "workshop_admin_land_type_product_list", [
                 (store_repeat_object, ":workShopIndex"),
                 (assign, "$current_workshop_type", ":workShopIndex"),
             ]],
            #
            #
            # ## 产品选择
            [anyone, "workshop_admin_land_type_product_list", [],
             "This is {s1},What kind of project are you want to choice?",
             "workshop_admin_land_type_product_list_choice", []],
            [anyone | plyr | repeat_for_100, "workshop_admin_land_type_product_list_choice", [
                (store_repeat_object, ":productIndex"),
                ## 获得该类型的商品数量
                (store_mul, ":workShopNameIndex", "$current_workshop_type", workShopPropertyCount),
                ## 工作间名称索引
                (val_add, ":workShopNameIndex", 1),
                ## 工作间最大产品数量索引(商品名称索引向后3位就是商品最大值的索引)
                (store_add, ":workShopProductMaxCountIndex", ":workShopNameIndex", 3),
                ## 获得该工作间最大产品数量
                (troop_get_slot, ":productMax", workShopListArray, ":workShopProductMaxCountIndex"),

                (is_between, ":productIndex", 0, ":productMax"),

                ## 当前物品索引 = 物品开始索引 + 当前索引
                ## 当前物品名索引 = 当前物品索引 * 物品属性个数 + 1

                ## 获得该工作间，第一个产品开始索引
                (store_add, ":productStartIndex", ":workShopProductMaxCountIndex", 1),
                (troop_get_slot, ":productFirstIndex", workShopListArray, ":productStartIndex"),
                (store_add, ":currentProductIndex", ":productFirstIndex", ":productIndex"),

                (store_mul, ":currentProductNameIndex", ":currentProductIndex", productPropertyCount),
                (val_add, ":currentProductNameIndex", 1),
                (troop_get_slot, ":productName", productListArray, ":currentProductNameIndex"),
                (store_add, ":currentProductPrice",":currentProductNameIndex", 3),
                (troop_get_slot, ":productPrice", productListArray, ":currentProductPrice"),
                (assign,reg0,":productPrice"),
                (store_add, ":currentProductOverhead", ":currentProductNameIndex", 4),
                (troop_get_slot, ":productOverhead", productListArray, ":currentProductOverhead"),
                (assign,reg1,":productOverhead"),
                ## 获得商品名称
                (str_store_string, s2, ":productName"),
            ], "{s2}({reg0})", "workshop_admin_land_type_product_list_choice_over", [
                (store_repeat_object, ":productIndex"),
                ## 获得该类型的商品数量
                (store_mul, ":workShopNameIndex", "$current_workshop_type", workShopPropertyCount),
                ## 工作间名称索引
                (val_add, ":workShopNameIndex", 1),
                ## 工作间最大产品数量索引(商品名称索引向后3位就是商品最大值的索引)
                (store_add, ":workShopProductMaxCountIndex", ":workShopNameIndex", 3),
                ## 获得该工作间最大产品数量
                (troop_get_slot, ":productMax", workShopListArray, ":workShopProductMaxCountIndex"),

                (is_between, ":productIndex", 0, ":productMax"),

                ## 当前物品索引 = 物品开始索引 + 当前索引
                ## 当前物品名索引 = 当前物品索引 * 物品属性个数 + 1

                ## 获得该工作间，第一个产品开始索引
                (store_add, ":productStartIndex", ":workShopProductMaxCountIndex", 1),
                (troop_get_slot, ":productFirstIndex", workShopListArray, ":productStartIndex"),
                (store_add, ":currentProductIndex", ":productFirstIndex", ":productIndex"),

                (store_mul, ":currentProductNameIndex", ":currentProductIndex", productPropertyCount),
                (val_add,":currentProductNameIndex",1),
                ## 物品名称
                (troop_get_slot, ":productName", productListArray, ":currentProductNameIndex"),
                ## 单个物品产量
                (store_add, ":currentProductQuantity", ":currentProductNameIndex", 1),
                (troop_get_slot, ":productQuantity", productListArray, ":currentProductQuantity"),
                (troop_set_slot, "$g_talk_troop", slot_troop_workshop_product_quantity, ":productQuantity"),
                ## 物品编号
                (store_add, ":currentProductItemIndex", ":currentProductNameIndex",2),
                (troop_get_slot, ":productType", productListArray, ":currentProductItemIndex"),
                (troop_set_slot, "$g_talk_troop", slot_troop_workshop_product_type, ":productType"),
                ## 物品价格
                (store_add, ":currentProductPrice", ":currentProductNameIndex", 3),
                (troop_get_slot, ":productPrice", productListArray, ":currentProductPrice"),
                (troop_set_slot, "$g_talk_troop", slot_troop_workshop_product_price, ":productPrice"),
                ## 单个物品开销
                (store_add, ":currentProductOverhead", ":currentProductNameIndex", 4),
                (troop_get_slot, ":productOverhead", productListArray, ":currentProductOverhead"),
                (troop_set_slot, "$g_talk_troop", slot_troop_workshop_product_overhead, ":productOverhead"),
             ]],

            # ## 产品选择结束
            [anyone, "workshop_admin_land_type_product_list_choice_over", [], "I'll do it well, boss!",
             "workshop_admin_land_type_product_list_choice_over_finish", []],
            [anyone | plyr, "workshop_admin_land_type_product_list_choice_over_finish", [], "Well done.","workshop_admin_start", []],
            ## 事项列表1：查看仓库
            [anyone|plyr, "workshop_admin_todo_list", [],"I want to check the warehouse.", "workshop_admin_start",[
               (change_screen_loot, "$g_talk_troop"),
             ]],
            ## 事项列表1：更改存储策略
            [anyone|plyr, "workshop_admin_todo_list", [],"I want to change the storage strategy.", "workshop_admin_storage_strategy_start",[]],
            [anyone, "workshop_admin_storage_strategy_start", [
                (troop_get_slot,":strategyType", "$g_talk_troop", slot_troop_workshop_storage_strategy),
                (store_add,":storageStrategyName","str_storage_strategy_1",":strategyType"),
                (str_store_string,s1,":storageStrategyName"),
            ],"The current strategy is {s1},What is our strategy?", "workshop_admin_storage_strategy_list",[]],
            [anyone|plyr|repeat_for_100, "workshop_admin_storage_strategy_list", [
                (store_repeat_object,":strategyType"),
                (is_between,":strategyType",0,3),
                (store_add,":storageStrategyName","str_storage_strategy_1",":strategyType"),
                (str_store_string,s2,":storageStrategyName"),
            ],"{s2}", "workshop_admin_storage_strategy_end",[
                (store_repeat_object,":strategyType"),
                (troop_set_slot, "$g_talk_troop", slot_troop_workshop_storage_strategy, ":strategyType"),
            ]],
            [anyone|plyr, "workshop_admin_storage_strategy_list", [], "Keep it up.", "workshop_admin_storage_strategy_end", []],
            [anyone, "workshop_admin_storage_strategy_end", [], "I see.", "workshop_admin_start", []],

            ## 事项列表：结束
            [anyone|plyr, "workshop_admin_todo_list", [], "forget it.", "close_window", []],
        ]),
        ("InitWalkers",[
            (str_store_party_name,s1,"$current_town"),
            (display_message,"@wellcome to {s1}"),
            (try_for_range, ":companion", companions_begin, companions_end),
                ## npc是管理员
                (troop_slot_eq,":companion",slot_troop_occupation,slto_workshop_admin),
                ## npc所在地是当前地
                (troop_slot_eq,":companion",slot_troop_cur_center,"$current_town"),
                ##(modify_visitors_at_site, ":cur_scene"),
                (set_visitor, 32, ":companion"),
                (str_store_troop_name,s2,":companion"),
                ##(display_message,"@place workshop admin {s2} in {s1}"),
            (try_end),
        ]),
        ## 玩家同意npc加入队伍的对话（如果不是管理员才显示）
        ("Prepend@dialogs|companion_was_dismissed&1:want_me_to_rejoin_your_company_%$&3:companion_rehire&4>2>#last",[
            (neg|troop_slot_eq,"$g_talk_troop",slot_troop_occupation,slto_workshop_admin),
        ]),
        ## 玩家同意npc加入队伍的对话（如果不是管理员才显示）
        ("Prepend@dialogs|companion_rehire&1:Welcome_back__my_friend%^&3:companion_recruit_signup_confirm&4>2>#last",[
            (neg|troop_slot_eq,"$g_talk_troop",slot_troop_occupation,slto_workshop_admin),
        ]),
        ## 玩家拒绝npc加入队伍的对话（如果不是管理员才显示）
        ("Prepend@dialogs|companion_rehire&1:Sorry__I_can_t_take%^&3:companion_rehire_refused&4>2>#last",[
            (neg|troop_slot_eq,"$g_talk_troop",slot_troop_occupation,slto_workshop_admin),
        ]),
        ("Append@strings", [
            ("storage_strategy_1", "sell_all"),
            ("storage_strategy_2", "keep_all"),
            ("storage_strategy_3", "sell_remain"),
        ]),
    ],
    "internationals":{
        "cns":{
            "dialogs":[
                "dlga_village_elder_talk:talk_elder_workshop_rent_land|我 想 租 一 块 地 建 一 个 工 作 间 。",
                "dlga_talk_elder_workshop_rent_land:talk_elder_workshop_admin_list_choice|谁 来 管 理 ？",
                "dlga_talk_elder_workshop_rent_land:close_window|你 已 经 有 管 理 员 了， 他 就 在 那 里 !",
                "dlga_talk_elder_workshop_admin_list_choice:talk_elder_workshop_rent_land_count|{s1} (统 御 {reg1}级 )",
                "dlga_talk_elder_workshop_admin_list_choice:close_window|我 没 有 合 适 的 人 选 。",
                "dlga_talk_elder_workshop_rent_land_count:talk_elder_workshop_rent_land_count_choice|你 想 租 多 少 块 地 ？",
                "dlga_talk_elder_workshop_rent_land_count_choice:close_window|买 {reg1}块 地 ( 花 费 {reg2}第 纳 尔)",
                "dlga_talk_elder_workshop_rent_land_count_choice:close_window.1|我 买 不 起 。",

                "dlga_companion_rehire:workshop_admin_start|让 我 们 讨 论 下 工 作 间 的 事 项 。",
                "dlga_workshop_admin_start:workshop_admin_todo_list|你 有 什 么 指 示 ?",
                "dlga_workshop_admin_todo_list:close_window|算 了 。",
                "dlga_workshop_admin_todo_list:workshop_admin_worksho_type_start|让 我 们 讨 论 下 工 作 间 的 的 类 型 。",
                "dlga_workshop_admin_worksho_type_start:workshop_admin_land_type|你 想 要 建 造 哪 种 工 作 间 ?",
                "dlga_workshop_admin_land_type_product_list:workshop_admin_land_type_product_list_choice|我 们 的 工 作 间 是 {s1} , 你 想 要 选 择 哪 种 产 品 ?",
                "dlga_workshop_admin_land_type_product_list_choice_over:workshop_admin_land_type_product_list_choice_over_finish|我 会 做 得 很 好 ， 老 板 ！",
                "dlga_workshop_admin_land_type_product_list_choice_over_finish:workshop_admin_start|很 好 。",
                "dlga_workshop_admin_todo_list:workshop_admin_start|我 想 检 查 下 仓 库 。",
                "dlga_workshop_admin_todo_list:workshop_admin_storage_strategy_start|我 想 改 变 下 存 储 策 略",
                "dlga_workshop_admin_storage_strategy_start:workshop_admin_storage_strategy_list|当 前 的 策 略 是 ”{s1}“ ，我 们 以 后 的 策 略 是 ？",
                "dlga_workshop_admin_storage_strategy_list:workshop_admin_storage_strategy_end|{s2}",
                "dlga_workshop_admin_storage_strategy_list:workshop_admin_storage_strategy_end.1|保 持 原 来 的 策 略 。",
                "dlga_workshop_admin_storage_strategy_end:close_window|我 明 白",
                "dlga_companion_rehire:close_window|没 事 。",
            ],
            "game_strings":[
                "str_storage_strategy_1|出 售 多 余 的",
                "str_storage_strategy_2|全 部 出 售 ",
                "str_storage_strategy_3|全 部 保 留 ",
            ],
            ## key超过20字符就无效
            "quick_strings":[
                "qstr_It_s_harvest_time_ag|系 统 提 示 ： 又 到 了 丰 收 的 季 节 ！",
                "qstr_{s0}_{s1}_Please_set|{s0}({s1})：请 选 择 工 作 间 的 产 品 类 型 ！",
                "qstr_{s0}_{s1}_products_{|{s0}({s1})：本 季 产 品 数 据 ， 收 获 {reg0}{s2}（ 出 售 数 量 {reg1} 土 地 数 量 {reg20} 存 储 数 量 {reg2} ）",
                "qstr_{s0}_{s1}_sell_amoun|{s0}({s1})：本 季 销 售 数 据 ， 销 量 {reg31}（ 基 础 销 量 {reg4} 交 易 技 能 {reg21} 管 理 员 加 成 {reg5} 成 本 开 销 {reg6}）",
                "qstr_next_quarterly_outpu|下 一 季 产 量 ， 产 量 ：{reg6} 管 理 技 能 ：{reg31} 管 理 员 加 成 ：{reg30}",
                "qstr_wellcome_to_{s1}|欢 迎 来 到 {s1}",
                "qstr_cost_{reg4}_for_{reg|花 费 {reg4} 购 买 {reg3} 块 地",
            ],
        }
    },
}


from smart_module.module_system.module_tableau_materials import tableaus
from smart_module.module_system.process_operations import *


def save_tableau_materials(variable_list,variable_uses,tag_uses,quick_strings,export_dir):
  ofile = open(export_dir + "tableau_materials.txt","w",encoding="utf-8")
  ofile.write("%d\n"%(len(tableaus)))
  for tableau in tableaus:
    ofile.write("tab_%s %d %s %d %d %d %d %d %d"%(tableau[0], tableau[1], tableau[2], tableau[3], tableau[4], tableau[5], tableau[6], tableau[7], tableau[8]))
    save_statement_block(ofile, 0, 1, tableau[9], variable_list, variable_uses, tag_uses, quick_strings)
    ofile.write("\n")
  ofile.close()

def save_python_header(src_dir):
  ofile = open(src_dir + "ID_tableau_materials.py","w",encoding="utf-8")
  for i_tableau in range(len(tableaus)):
    ofile.write("tableau_%s = %d\n"%(tableaus[i_tableau][0],i_tableau))
  ofile.close()


def processTableauMaterials(context):
  print( "Exporting tableau materials data...")

  configParser = context.configParser
  src_dir = configParser.getSrcDir()
  export_dir = configParser.getExportDir()

  save_python_header(src_dir)
  variable_uses = []
  variables = load_variables(export_dir, variable_uses)
  tag_uses = load_tag_uses(export_dir)
  quick_strings = load_quick_strings(export_dir)
  save_tableau_materials(variables,variable_uses,tag_uses,quick_strings,export_dir)
  save_variables(export_dir,variables,variable_uses)
  save_tag_uses(export_dir, tag_uses)
  save_quick_strings(export_dir,quick_strings)



from smart_module.module_system.module_dialogs import *
from smart_module.module_system.module_triggers import triggers

from smart_module.module_system.process_operations import *


speaker_pos = 0
ipt_token_pos = 1
sentence_conditions_pos = 2
text_pos = 3
opt_token_pos = 4
sentence_consequences_pos = 5
sentence_voice_over_pos = 6



#-------------------------------------------------------

def save_dialog_states(dialog_states,export_dir):
  file = open(export_dir + "dialog_states.txt","w",encoding="utf-8")
  for dialog_state in dialog_states:
    file.write("%s\n"%dialog_state)
  file.close()


#def compile_variables(cookies_list):
#  for trigger in triggers:
#    for consequence in trigger[trigger_consequences_pos]:
#      compile_statement(consequence,cookies_list)
#  for sentence in sentences:
#    for consequence in sentence[sentence_consequences_pos]:
#      compile_statement(consequence,cookies_list)
#  for trigger in triggers:
#    for condition in trigger[trigger_conditions_pos]:
#      compile_statement(condition,cookies_list)
#  for sentence in sentences:
#    for condition in sentence[sentence_conditions_pos]:
#      compile_statement(condition,cookies_list)
#  return cookies_list

def save_triggers(variable_list,variable_uses,triggers,tag_uses,quick_strings,export_dir):
  file = open(export_dir + "triggers.txt","w",encoding="utf-8")
  file.write("triggersfile version 1\n")
  file.write("%d\n"%len(triggers))
  for i in range(len(triggers)):
    trigger = triggers[i]
    file.write("%f %f %f "%(trigger[trigger_check_pos],trigger[trigger_delay_pos],trigger[trigger_rearm_pos]))
    save_statement_block(file,0,1,trigger[trigger_conditions_pos]  , variable_list, variable_uses,tag_uses,quick_strings)
    save_statement_block(file,0,1,trigger[trigger_consequences_pos], variable_list, variable_uses,tag_uses,quick_strings)
#    for condition in trigger[trigger_conditions_pos]:
#      save_operation(file,condition,variable_list)
#    file.write(" %d "%(len(trigger[trigger_consequences_pos])))
#    for consequence in trigger[trigger_consequences_pos]:
#      save_operation(file,consequence,variable_list)
    file.write("\n")
  file.close()


#=================================================================
def compile_sentence_tokens(sentences,export_dir):
  input_tokens = []
  output_tokens = []
  dialog_states = ["start","party_encounter","prisoner_liberated","enemy_defeated","party_relieved","event_triggered","close_window","trade","exchange_members", "trade_prisoners","buy_mercenaries","view_char","training","member_chat","prisoner_chat"]
  dialog_state_usages = [1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1]
  for sentence in sentences:
    output_token_id = -1
    output_token = sentence[opt_token_pos]
    found = 0
    for i_t in range(len(dialog_states)):
      if output_token == dialog_states[i_t]:
        output_token_id = i_t
        found = 1
        break
    if not found:
      dialog_states.append(output_token)
      dialog_state_usages.append(0)
      output_token_id = len(dialog_states) - 1
    output_tokens.append(output_token_id)
  for sentence in sentences:
    input_token_id = -1
    input_token = sentence[ipt_token_pos]
    found = 0
    for i_t in range(len(dialog_states)):
      if input_token == dialog_states[i_t]:
        input_token_id = i_t
        dialog_state_usages[i_t] = dialog_state_usages[i_t] + 1
        found = 1
        break
    if not found:
      print( sentence[ipt_token_pos])
      print( sentence[text_pos])
      print( sentence[opt_token_pos])
      print( "**********************************************************************************")
      print( "ERROR: INPUT TOKEN NOT FOUND:" + input_token)
      print( "**********************************************************************************")
      print( "**********************************************************************************")
    input_tokens.append(input_token_id)
  save_dialog_states(dialog_states,export_dir)
  for i_t in range(len(dialog_states)):
    if dialog_state_usages[i_t] == 0:
      print( "ERROR: Output token not found: " + dialog_states[i_t])
  return (input_tokens, output_tokens)

def create_auto_id(sentence,auto_ids):
    text = convert_to_identifier(sentence[text_pos])
    done = 0
    i = 20
    lt = len(text)
    if (i > lt):
      i  = lt
    auto_id = "dlga_" + text[0:i]
    done = 0
    if auto_id in auto_ids and (auto_ids[auto_id] == text):
      done = 1
    while (i <= lt) and not done:
      auto_id = "dlga_" + text[0:i]
      if auto_id in auto_ids:
        if auto_ids[auto_id] == text:
          done = 1
        else:
          i += 1
      else:      
        done = 1
        auto_ids[auto_id] = text
    if not done:
      number = 1
      new_auto_id = auto_id + str(number)
      while new_auto_id in auto_ids:
        number += 1
        new_auto_id = auto_id + str(number)
      auto_id = new_auto_id
      auto_ids[auto_id] = text
    return auto_id
  
def create_auto_id2(sentence,auto_ids):
    text = sentence[text_pos]
    token_ipt = convert_to_identifier(sentence[ipt_token_pos])
    token_opt = convert_to_identifier(sentence[opt_token_pos])
    done = 0
    auto_id = "dlga_" + token_ipt + ":" + token_opt
    done = 0
    if not auto_id in auto_ids:
      done = 1
    else:
      if auto_id in auto_ids and (auto_ids[auto_id] == text):
        done = 1
    if not done:
      number = 1
      new_auto_id = auto_id + "." + str(number)
      while new_auto_id in auto_ids:
        number += 1
        new_auto_id = auto_id + "." + str(number)
      auto_id = new_auto_id
    auto_ids[auto_id] = text
    return auto_id
 
def save_sentences(variable_list,variable_uses,sentences,tag_uses,quick_strings,input_states,output_states,export_dir):
  file = open(export_dir + "conversation.txt","w",encoding="utf-8")
  file.write("dialogsfile version 2\n")
  file.write("%d\n"%len(sentences))
  # Create an empty dictionary
  auto_ids = {}
  for i in range(len(sentences)):
    sentence = sentences[i]
    try:
      dialog_id = create_auto_id2(sentence,auto_ids)
      file.write("%s %d %d "%(dialog_id,sentence[speaker_pos],input_states[i]))
      save_statement_block(file, 0, 1, sentence[sentence_conditions_pos], variable_list,variable_uses,tag_uses,quick_strings)

      file.write("%s "%(sentence[text_pos].replace(" ","_")))
      if (len(sentence[text_pos]) == 0):
        file.write("NO_TEXT ")
      file.write(" %d "%(output_states[i]))
      save_statement_block(file, 0, 1, sentence[sentence_consequences_pos], variable_list,variable_uses,tag_uses,quick_strings)
      if (len(sentence) > sentence_voice_over_pos):
        file.write("%s "%sentence[sentence_voice_over_pos])
      else:
        file.write("NO_VOICEOVER ")
      file.write("\n")
    except:
      print( "Error in dialog line:")
      print( sentence)
  file.close()

# Registered cookies is a list which enables the order of cookies to remain fixed across changes.
# In order to remove cookies not used anymore, edit the cookies_registery.py and remove all entries.

def processDialogs(context):
  print("exporting triggers...")

  configParser = context.configParser
  src_dir = configParser.getSrcDir()
  export_dir = configParser.getExportDir()

  variable_uses = []
  variables = load_variables(export_dir, variable_uses)
  tag_uses = load_tag_uses(export_dir)
  quick_strings = load_quick_strings(export_dir)
  # compile_variables(variables)
  save_triggers(variables, variable_uses, triggers, tag_uses, quick_strings,export_dir)
  print("exporting dialogs...")
  (input_states, output_states) = compile_sentence_tokens(dialogs,export_dir)
  save_sentences(variables, variable_uses, dialogs, tag_uses, quick_strings, input_states, output_states,export_dir)
  save_variables(export_dir, variables, variable_uses)
  save_tag_uses(export_dir, tag_uses)
  save_quick_strings(export_dir, quick_strings)
  # print "finished."



mst_town_default = 0
mst_conversation_encounter = 1
mst_town_center = 2
mst_village_center = 3
mst_bandits_at_night = 4
mst_village_training = 5
mst_visit_town_castle = 6
mst_back_alley_kill_local_merchant = 7
mst_back_alley_revolt = 8
mst_lead_charge = 9
mst_village_attack_bandits = 10
mst_village_raid = 11
mst_besiege_inner_battle_castle = 12
mst_besiege_inner_battle_town_center = 13
mst_castle_attack_walls_defenders_sally = 14
mst_castle_attack_walls_belfry = 15
mst_castle_attack_walls_ladder = 16
mst_castle_visit = 17
mst_training_ground_trainer_talk = 18
mst_training_ground_trainer_training = 19
mst_training_ground_training = 20
mst_sneak_caught_fight = 21
mst_ai_training = 22
mst_camera_test = 23
mst_arena_melee_fight = 24
mst_arena_challenge_fight = 25
mst_duel_with_lord = 26
mst_wedding = 27
mst_tutorial_training_ground = 28
mst_tutorial_1 = 29
mst_tutorial_2 = 30
mst_tutorial_3 = 31
mst_tutorial_3_2 = 32
mst_tutorial_4 = 33
mst_tutorial_5 = 34
mst_quick_battle_battle = 35
mst_quick_battle_siege = 36
mst_multiplayer_dm = 37
mst_multiplayer_tdm = 38
mst_multiplayer_hq = 39
mst_multiplayer_cf = 40
mst_multiplayer_sg = 41
mst_multiplayer_bt = 42
mst_multiplayer_fd = 43
mst_bandit_lair = 44
mst_alley_fight = 45
mst_meeting_merchant = 46
mst_town_fight = 47
mst_multiplayer_duel = 48



from smart_module.module_system.module_presentations import presentations
from smart_module.module_system.process_operations import *


def save_presentations(variable_list,variable_uses,tag_uses,quick_strings,export_dir):
  ofile = open(export_dir + "presentations.txt","w",encoding="utf-8")
  ofile.write("presentationsfile version 1\n")
  ofile.write(" %d\n"%(len(presentations)))
  for presentation in presentations:
    ofile.write("prsnt_%s %d %d "%(presentation[0], presentation[1], presentation[2]))
    save_simple_triggers(ofile,presentation[3], variable_list,variable_uses,tag_uses,quick_strings)
    ofile.write("\n")
  ofile.close()


def save_python_header(src_dir):
  file = open(src_dir + "ID_presentations.py","w",encoding="utf-8")
  for i_presentation in range(len(presentations)):
    file.write("prsnt_%s = %d\n"%(presentations[i_presentation][0],i_presentation))
  file.close()


def processPresentations(context):
  print( "Exporting presentations...")

  configParser = context.configParser
  src_dir = configParser.getSrcDir()
  export_dir = configParser.getExportDir()

  save_python_header(src_dir)
  variable_uses = []
  variables = load_variables(export_dir,variable_uses)
  tag_uses = load_tag_uses(export_dir)
  quick_strings = load_quick_strings(export_dir)
  save_presentations(variables,variable_uses,tag_uses,quick_strings,export_dir)
  save_variables(export_dir,variables,variable_uses)
  save_tag_uses(export_dir,tag_uses)
  save_quick_strings(export_dir,quick_strings)



ground_gray_stone = 0
ground_brown_stone = 1
ground_turf = 2
ground_steppe = 3
ground_snow = 4
ground_earth = 5
ground_desert = 6
ground_forest = 7
ground_pebbles = 8
ground_village = 9
ground_path = 10





scn_random_scene = 0
scn_conversation_scene = 1
scn_water = 2
scn_random_scene_steppe = 3
scn_random_scene_plain = 4
scn_random_scene_snow = 5
scn_random_scene_desert = 6
scn_random_scene_steppe_forest = 7
scn_random_scene_plain_forest = 8
scn_random_scene_snow_forest = 9
scn_random_scene_desert_forest = 10
scn_camp_scene = 11
scn_camp_scene_horse_track = 12
scn_four_ways_inn = 13
scn_test_scene = 14
scn_quick_battle_1 = 15
scn_quick_battle_2 = 16
scn_quick_battle_3 = 17
scn_quick_battle_4 = 18
scn_quick_battle_5 = 19
scn_quick_battle_6 = 20
scn_quick_battle_7 = 21
scn_salt_mine = 22
scn_novice_ground = 23
scn_zendar_arena = 24
scn_dhorak_keep = 25
scn_reserved4 = 26
scn_reserved5 = 27
scn_reserved6 = 28
scn_reserved7 = 29
scn_reserved8 = 30
scn_reserved9 = 31
scn_reserved10 = 32
scn_reserved11 = 33
scn_reserved12 = 34
scn_training_ground = 35
scn_tutorial_1 = 36
scn_tutorial_2 = 37
scn_tutorial_3 = 38
scn_tutorial_4 = 39
scn_tutorial_5 = 40
scn_training_ground_horse_track_1 = 41
scn_training_ground_horse_track_2 = 42
scn_training_ground_horse_track_3 = 43
scn_training_ground_horse_track_4 = 44
scn_training_ground_horse_track_5 = 45
scn_training_ground_ranged_melee_1 = 46
scn_training_ground_ranged_melee_2 = 47
scn_training_ground_ranged_melee_3 = 48
scn_training_ground_ranged_melee_4 = 49
scn_training_ground_ranged_melee_5 = 50
scn_zendar_center = 51
scn_the_happy_boar = 52
scn_zendar_merchant = 53
scn_town_1_center = 54
scn_town_2_center = 55
scn_town_3_center = 56
scn_town_4_center = 57
scn_town_5_center = 58
scn_town_6_center = 59
scn_town_7_center = 60
scn_town_8_center = 61
scn_town_9_center = 62
scn_town_10_center = 63
scn_town_11_center = 64
scn_town_12_center = 65
scn_town_13_center = 66
scn_town_14_center = 67
scn_town_15_center = 68
scn_town_16_center = 69
scn_town_17_center = 70
scn_town_18_center = 71
scn_town_19_center = 72
scn_town_20_center = 73
scn_town_21_center = 74
scn_town_22_center = 75
scn_town_1_castle = 76
scn_town_2_castle = 77
scn_town_3_castle = 78
scn_town_4_castle = 79
scn_town_5_castle = 80
scn_town_6_castle = 81
scn_town_7_castle = 82
scn_town_8_castle = 83
scn_town_9_castle = 84
scn_town_10_castle = 85
scn_town_11_castle = 86
scn_town_12_castle = 87
scn_town_13_castle = 88
scn_town_14_castle = 89
scn_town_15_castle = 90
scn_town_16_castle = 91
scn_town_17_castle = 92
scn_town_18_castle = 93
scn_town_19_castle = 94
scn_town_20_castle = 95
scn_town_21_castle = 96
scn_town_22_castle = 97
scn_town_1_tavern = 98
scn_town_2_tavern = 99
scn_town_3_tavern = 100
scn_town_4_tavern = 101
scn_town_5_tavern = 102
scn_town_6_tavern = 103
scn_town_7_tavern = 104
scn_town_8_tavern = 105
scn_town_9_tavern = 106
scn_town_10_tavern = 107
scn_town_11_tavern = 108
scn_town_12_tavern = 109
scn_town_13_tavern = 110
scn_town_14_tavern = 111
scn_town_15_tavern = 112
scn_town_16_tavern = 113
scn_town_17_tavern = 114
scn_town_18_tavern = 115
scn_town_19_tavern = 116
scn_town_20_tavern = 117
scn_town_21_tavern = 118
scn_town_22_tavern = 119
scn_town_1_store = 120
scn_town_2_store = 121
scn_town_3_store = 122
scn_town_4_store = 123
scn_town_5_store = 124
scn_town_6_store = 125
scn_town_7_store = 126
scn_town_8_store = 127
scn_town_9_store = 128
scn_town_10_store = 129
scn_town_11_store = 130
scn_town_12_store = 131
scn_town_13_store = 132
scn_town_14_store = 133
scn_town_15_store = 134
scn_town_16_store = 135
scn_town_17_store = 136
scn_town_18_store = 137
scn_town_19_store = 138
scn_town_20_store = 139
scn_town_21_store = 140
scn_town_22_store = 141
scn_town_1_arena = 142
scn_town_2_arena = 143
scn_town_3_arena = 144
scn_town_4_arena = 145
scn_town_5_arena = 146
scn_town_6_arena = 147
scn_town_7_arena = 148
scn_town_8_arena = 149
scn_town_9_arena = 150
scn_town_10_arena = 151
scn_town_11_arena = 152
scn_town_12_arena = 153
scn_town_13_arena = 154
scn_town_14_arena = 155
scn_town_15_arena = 156
scn_town_16_arena = 157
scn_town_17_arena = 158
scn_town_18_arena = 159
scn_town_19_arena = 160
scn_town_20_arena = 161
scn_town_21_arena = 162
scn_town_22_arena = 163
scn_town_1_prison = 164
scn_town_2_prison = 165
scn_town_3_prison = 166
scn_town_4_prison = 167
scn_town_5_prison = 168
scn_town_6_prison = 169
scn_town_7_prison = 170
scn_town_8_prison = 171
scn_town_9_prison = 172
scn_town_10_prison = 173
scn_town_11_prison = 174
scn_town_12_prison = 175
scn_town_13_prison = 176
scn_town_14_prison = 177
scn_town_15_prison = 178
scn_town_16_prison = 179
scn_town_17_prison = 180
scn_town_18_prison = 181
scn_town_19_prison = 182
scn_town_20_prison = 183
scn_town_21_prison = 184
scn_town_22_prison = 185
scn_town_1_walls = 186
scn_town_2_walls = 187
scn_town_3_walls = 188
scn_town_4_walls = 189
scn_town_5_walls = 190
scn_town_6_walls = 191
scn_town_7_walls = 192
scn_town_8_walls = 193
scn_town_9_walls = 194
scn_town_10_walls = 195
scn_town_11_walls = 196
scn_town_12_walls = 197
scn_town_13_walls = 198
scn_town_14_walls = 199
scn_town_15_walls = 200
scn_town_16_walls = 201
scn_town_17_walls = 202
scn_town_18_walls = 203
scn_town_19_walls = 204
scn_town_20_walls = 205
scn_town_21_walls = 206
scn_town_22_walls = 207
scn_town_1_alley = 208
scn_town_2_alley = 209
scn_town_3_alley = 210
scn_town_4_alley = 211
scn_town_5_alley = 212
scn_town_6_alley = 213
scn_town_7_alley = 214
scn_town_8_alley = 215
scn_town_9_alley = 216
scn_town_10_alley = 217
scn_town_11_alley = 218
scn_town_12_alley = 219
scn_town_13_alley = 220
scn_town_14_alley = 221
scn_town_15_alley = 222
scn_town_16_alley = 223
scn_town_17_alley = 224
scn_town_18_alley = 225
scn_town_19_alley = 226
scn_town_20_alley = 227
scn_town_21_alley = 228
scn_town_22_alley = 229
scn_castle_1_exterior = 230
scn_castle_1_interior = 231
scn_castle_1_prison = 232
scn_castle_2_exterior = 233
scn_castle_2_interior = 234
scn_castle_2_prison = 235
scn_castle_3_exterior = 236
scn_castle_3_interior = 237
scn_castle_3_prison = 238
scn_castle_4_exterior = 239
scn_castle_4_interior = 240
scn_castle_4_prison = 241
scn_castle_5_exterior = 242
scn_castle_5_interior = 243
scn_castle_5_prison = 244
scn_castle_6_exterior = 245
scn_castle_6_interior = 246
scn_castle_6_prison = 247
scn_castle_7_exterior = 248
scn_castle_7_interior = 249
scn_castle_7_prison = 250
scn_castle_8_exterior = 251
scn_castle_8_interior = 252
scn_castle_8_prison = 253
scn_castle_9_exterior = 254
scn_castle_9_interior = 255
scn_castle_9_prison = 256
scn_castle_10_exterior = 257
scn_castle_10_interior = 258
scn_castle_10_prison = 259
scn_castle_11_exterior = 260
scn_castle_11_interior = 261
scn_castle_11_prison = 262
scn_castle_12_exterior = 263
scn_castle_12_interior = 264
scn_castle_12_prison = 265
scn_castle_13_exterior = 266
scn_castle_13_interior = 267
scn_castle_13_prison = 268
scn_castle_14_exterior = 269
scn_castle_14_interior = 270
scn_castle_14_prison = 271
scn_castle_15_exterior = 272
scn_castle_15_interior = 273
scn_castle_15_prison = 274
scn_castle_16_exterior = 275
scn_castle_16_interior = 276
scn_castle_16_prison = 277
scn_castle_17_exterior = 278
scn_castle_17_interior = 279
scn_castle_17_prison = 280
scn_castle_18_exterior = 281
scn_castle_18_interior = 282
scn_castle_18_prison = 283
scn_castle_19_exterior = 284
scn_castle_19_interior = 285
scn_castle_19_prison = 286
scn_castle_20_exterior = 287
scn_castle_20_interior = 288
scn_castle_20_prison = 289
scn_castle_21_exterior = 290
scn_castle_21_interior = 291
scn_castle_21_prison = 292
scn_castle_22_exterior = 293
scn_castle_22_interior = 294
scn_castle_22_prison = 295
scn_castle_23_exterior = 296
scn_castle_23_interior = 297
scn_castle_23_prison = 298
scn_castle_24_exterior = 299
scn_castle_24_interior = 300
scn_castle_24_prison = 301
scn_castle_25_exterior = 302
scn_castle_25_interior = 303
scn_castle_25_prison = 304
scn_castle_26_exterior = 305
scn_castle_26_interior = 306
scn_castle_26_prison = 307
scn_castle_27_exterior = 308
scn_castle_27_interior = 309
scn_castle_27_prison = 310
scn_castle_28_exterior = 311
scn_castle_28_interior = 312
scn_castle_28_prison = 313
scn_castle_29_exterior = 314
scn_castle_29_interior = 315
scn_castle_29_prison = 316
scn_castle_30_exterior = 317
scn_castle_30_interior = 318
scn_castle_30_prison = 319
scn_castle_31_exterior = 320
scn_castle_31_interior = 321
scn_castle_31_prison = 322
scn_castle_32_exterior = 323
scn_castle_32_interior = 324
scn_castle_32_prison = 325
scn_castle_33_exterior = 326
scn_castle_33_interior = 327
scn_castle_33_prison = 328
scn_castle_34_exterior = 329
scn_castle_34_interior = 330
scn_castle_34_prison = 331
scn_castle_35_exterior = 332
scn_castle_35_interior = 333
scn_castle_35_prison = 334
scn_castle_36_exterior = 335
scn_castle_36_interior = 336
scn_castle_36_prison = 337
scn_castle_37_exterior = 338
scn_castle_37_interior = 339
scn_castle_37_prison = 340
scn_castle_38_exterior = 341
scn_castle_38_interior = 342
scn_castle_38_prison = 343
scn_castle_39_exterior = 344
scn_castle_39_interior = 345
scn_castle_39_prison = 346
scn_castle_40_exterior = 347
scn_castle_40_interior = 348
scn_castle_40_prison = 349
scn_castle_41_exterior = 350
scn_castle_41_interior = 351
scn_castle_41_prison = 352
scn_castle_42_exterior = 353
scn_castle_42_interior = 354
scn_castle_42_prison = 355
scn_castle_43_exterior = 356
scn_castle_43_interior = 357
scn_castle_43_prison = 358
scn_castle_44_exterior = 359
scn_castle_44_interior = 360
scn_castle_44_prison = 361
scn_castle_45_exterior = 362
scn_castle_45_interior = 363
scn_castle_45_prison = 364
scn_castle_46_exterior = 365
scn_castle_46_interior = 366
scn_castle_46_prison = 367
scn_castle_47_exterior = 368
scn_castle_47_interior = 369
scn_castle_47_prison = 370
scn_castle_48_exterior = 371
scn_castle_48_interior = 372
scn_castle_48_prison = 373
scn_village_1 = 374
scn_village_2 = 375
scn_village_3 = 376
scn_village_4 = 377
scn_village_5 = 378
scn_village_6 = 379
scn_village_7 = 380
scn_village_8 = 381
scn_village_9 = 382
scn_village_10 = 383
scn_village_11 = 384
scn_village_12 = 385
scn_village_13 = 386
scn_village_14 = 387
scn_village_15 = 388
scn_village_16 = 389
scn_village_17 = 390
scn_village_18 = 391
scn_village_19 = 392
scn_village_20 = 393
scn_village_21 = 394
scn_village_22 = 395
scn_village_23 = 396
scn_village_24 = 397
scn_village_25 = 398
scn_village_26 = 399
scn_village_27 = 400
scn_village_28 = 401
scn_village_29 = 402
scn_village_30 = 403
scn_village_31 = 404
scn_village_32 = 405
scn_village_33 = 406
scn_village_34 = 407
scn_village_35 = 408
scn_village_36 = 409
scn_village_37 = 410
scn_village_38 = 411
scn_village_39 = 412
scn_village_40 = 413
scn_village_41 = 414
scn_village_42 = 415
scn_village_43 = 416
scn_village_44 = 417
scn_village_45 = 418
scn_village_46 = 419
scn_village_47 = 420
scn_village_48 = 421
scn_village_49 = 422
scn_village_50 = 423
scn_village_51 = 424
scn_village_52 = 425
scn_village_53 = 426
scn_village_54 = 427
scn_village_55 = 428
scn_village_56 = 429
scn_village_57 = 430
scn_village_58 = 431
scn_village_59 = 432
scn_village_60 = 433
scn_village_61 = 434
scn_village_62 = 435
scn_village_63 = 436
scn_village_64 = 437
scn_village_65 = 438
scn_village_66 = 439
scn_village_67 = 440
scn_village_68 = 441
scn_village_69 = 442
scn_village_70 = 443
scn_village_71 = 444
scn_village_72 = 445
scn_village_73 = 446
scn_village_74 = 447
scn_village_75 = 448
scn_village_76 = 449
scn_village_77 = 450
scn_village_78 = 451
scn_village_79 = 452
scn_village_80 = 453
scn_village_81 = 454
scn_village_82 = 455
scn_village_83 = 456
scn_village_84 = 457
scn_village_85 = 458
scn_village_86 = 459
scn_village_87 = 460
scn_village_88 = 461
scn_village_89 = 462
scn_village_90 = 463
scn_village_91 = 464
scn_village_92 = 465
scn_village_93 = 466
scn_village_94 = 467
scn_village_95 = 468
scn_village_96 = 469
scn_village_97 = 470
scn_village_98 = 471
scn_village_99 = 472
scn_village_100 = 473
scn_village_101 = 474
scn_village_102 = 475
scn_village_103 = 476
scn_village_104 = 477
scn_village_105 = 478
scn_village_106 = 479
scn_village_107 = 480
scn_village_108 = 481
scn_village_109 = 482
scn_village_110 = 483
scn_field_1 = 484
scn_field_2 = 485
scn_field_3 = 486
scn_field_4 = 487
scn_field_5 = 488
scn_test2 = 489
scn_test3 = 490
scn_multi_scene_1 = 491
scn_multi_scene_2 = 492
scn_multi_scene_3 = 493
scn_multi_scene_4 = 494
scn_multi_scene_5 = 495
scn_multi_scene_6 = 496
scn_multi_scene_7 = 497
scn_multi_scene_8 = 498
scn_multi_scene_9 = 499
scn_multi_scene_10 = 500
scn_multi_scene_11 = 501
scn_multi_scene_12 = 502
scn_multi_scene_13 = 503
scn_multi_scene_14 = 504
scn_multi_scene_15 = 505
scn_multi_scene_16 = 506
scn_multi_scene_17 = 507
scn_multi_scene_18 = 508
scn_multi_scene_19 = 509
scn_multi_scene_20 = 510
scn_random_multi_plain_medium = 511
scn_random_multi_plain_large = 512
scn_random_multi_steppe_medium = 513
scn_random_multi_steppe_large = 514
scn_multiplayer_maps_end = 515
scn_wedding = 516
scn_lair_steppe_bandits = 517
scn_lair_taiga_bandits = 518
scn_lair_desert_bandits = 519
scn_lair_forest_bandits = 520
scn_lair_mountain_bandits = 521
scn_lair_sea_raiders = 522
scn_quick_battle_scene_1 = 523
scn_quick_battle_scene_2 = 524
scn_quick_battle_scene_3 = 525
scn_quick_battle_scene_4 = 526
scn_quick_battle_scene_5 = 527
scn_quick_battle_maps_end = 528
scn_tutorial_training_ground = 529
scn_town_1_room = 530
scn_town_5_room = 531
scn_town_6_room = 532
scn_town_8_room = 533
scn_town_10_room = 534
scn_town_19_room = 535
scn_meeting_scene_steppe = 536
scn_meeting_scene_plain = 537
scn_meeting_scene_snow = 538
scn_meeting_scene_desert = 539
scn_meeting_scene_steppe_forest = 540
scn_meeting_scene_plain_forest = 541
scn_meeting_scene_snow_forest = 542
scn_meeting_scene_desert_forest = 543
scn_enterprise_tannery = 544
scn_enterprise_winery = 545
scn_enterprise_mill = 546
scn_enterprise_smithy = 547
scn_enterprise_dyeworks = 548
scn_enterprise_linen_weavery = 549
scn_enterprise_wool_weavery = 550
scn_enterprise_brewery = 551
scn_enterprise_oil_press = 552
